# type: ignore

from dataclasses import dataclass


@dataclass
class Badge:
    """Achievement Badges associated with a Participant or Team.

    For Donor Drive api information: https://github.com/DonorDrive/PublicAPI/blob/master/resources/badges.md

    :param badge_code: A unique identifier for this badge.
    :type: str
    :param badge_image_url: The URL for the image associated with this badge
    :type: str
    :param description: The description of this badge
    :type: str
    :param title: The title of this badge
    :type: str
    :param unlocked_date_utc: The date this badge was unlocked by the Participant or Team
    :type: str
    """
    badge_code: str
    badge_image_url: str
    description: str
    title: str
    unlocked_date_utc: str

    @staticmethod
    def create_badge(json_data: dict):
        """Create a badge based on the JSON data.

        :param json_data: The JSON data for this badge.
        :type: dict
        :returns: a Badge object
        """
        return Badge(json_data.get("badgeCode"), json_data.get("badgeImageURL"), json_data.get("description"),
                     json_data.get('title'), json_data.get("unlockedDateUTC"))



from eldonationtracker.api import badge
import json


def test_badge_creation():
    """Test creation of a badge ojbect."""
    example_json = """[
{
    "description": "Raised 100 dollars!",
    "title": "100 Club Badge",
    "unlockedDateUTC": "2019-10-30T18:01:23.430+0000",
    "badgeImageURL": "http://assets.donordrive.com/try/images/$event508$/badge_2F7819D3_C019_3C7D_B9D716687CEEC0A5.png",
    "badgeCode": "100-club-badge"
  },
  {
    "description": "Sent 25 donation invite emails!",
    "title": "Enthusiastic Participant Badge",
    "unlockedDateUTC": "2019-09-18T15:47:39.107+0000",
    "badgeImageURL": "https://assets.donordrive.com/try/images/$event508$/badge_DCB0A883_BC0A_97DB_639B4D7BFDEC638E.png",
    "badgeCode": "enthusiastic-participant-badge"
  }
]"""
    example_json_as_json = json.loads(example_json)
    test_badges = [badge.Badge.create_badge(a_badge) for a_badge in example_json_as_json]
    assert test_badges[0].badge_code == "100-club-badge"
    assert test_badges[0].badge_image_url == "http://assets.donordrive.com/try/images/$event508$/badge_2F7819D3_C019_3C7D_B9D716687CEEC0A5.png"
    assert test_badges[0].description == "Raised 100 dollars!"
    assert test_badges[0].title == "100 Club Badge"
    assert test_badges[0].unlocked_date_utc == "2019-10-30T18:01:23.430+0000"
    assert test_badges[1].badge_code == "enthusiastic-participant-badge"
    assert test_badges[1].badge_image_url == "https://assets.donordrive.com/try/images/$event508$/badge_DCB0A883_BC0A_97DB_639B4D7BFDEC638E.png"
    assert test_badges[1].description == "Sent 25 donation invite emails!"
    assert test_badges[1].title == "Enthusiastic Participant Badge"
    assert test_badges[1].unlocked_date_utc == "2019-09-18T15:47:39.107+0000"



"""Contains classes pertaining to teams."""
import logging
from rich import print
from rich.logging import RichHandler
from typing import Tuple, List

from donordrivepython.api import comms as donor_drive_comms
from donordrivepython.api.badge import Badge  # type: ignore
from donordrivepython.api.team_participant import TeamParticipant
from donordrivepython.api import donation, activity

# logging
team_log = logging.getLogger("Team:")
team_log.setLevel(logging.INFO)


class Team:
    """Hold Team api Data."""

    def __init__(self, team_id: str, output_folder: str, currency_symbol: str, donors_to_display: str,
                 base_api_url: str):
        """Set the team variables.

        :param team_id: The team's ID in the api
        :type team_id: str
        :param output_folder: The folder for the output text files
        :type output_folder: str
        :param currency_symbol: for formatting text
        :type currency_symbol: str
        :param donors_to_display: the number of donors to write out to the text files
        :type donors_to_display: str
        """
        self._team_id: str = team_id
        # urls
        self._team_url_base: str = f"{base_api_url}/teams/"
        self._team_url: str = f"{self.team_url_base}{team_id}"
        self._team_participant_url: str = f"{self.team_url}/participants"
        self._team_donation_url: str = f"{self.team_url}/donations"
        # misc
        self._output_folder: str = output_folder
        self._currency_symbol: str = currency_symbol
        self._donors_to_display: str = donors_to_display
        # team info
        self._team_info: dict = {}  # a dictionary to values for output to text files
        self._team_goal: int = 0
        self._team_captain: str = ""
        self._total_raised: int = 0
        self._num_donations: int = 0
        self._team_avatar_image: str = ''
        # donor info
        self._participant_calculation_dict: dict = {}  # dictionary holding output for txt files
        self._top_5_participant_list: List[TeamParticipant] = []  # list: top 5 team participants by amount donated.
        self._participant_list: List[TeamParticipant] = []  # A list of the most recent participants
        # donation info
        self._donation_list: List[donation.Donation] = []
        self._donation_formatted_output: dict = {'Team_LastDonationNameAmnt': "No Donations Yet",
                                                 'Team_lastNDonationNameAmts': "No Donations Yet",
                                                 'Team_lastNDonationNameAmtsMessage': "No Donations Yet",
                                                 'Team_lastNDonationNameAmtsMessageHorizontal': "No Donations Yet",
                                                 'Team_lastNDonationNameAmtsHorizontal': "No Donations Yet"}
        # other api endpoints
        self._badge_url: str = f"{self.team_url}/badges"
        self._badges: list[Badge] = []
        self._activity_url: str = f"{self.team_url}/activity"
        self._activity_list: list[activity.Activity] = []

    @property
    def team_id(self) -> str:
        """The team's ID in the api."""
        return self._team_id

    @property
    def team_url_base(self) -> str:
        """The donor drive endpoint for the teams."""
        return self._team_url_base

    @property
    def team_url(self) -> str:
        """URL to the team JSON api."""
        return self._team_url

    @property
    def team_participant_url(self) -> str:
        """URL to the JSON api for participants in the team."""
        return self._team_participant_url

    @property
    def team_donation_url(self) -> str:
        """URL to the JSON api for donations to the team."""
        return self._team_donation_url

    @property
    def output_folder(self) -> str:
        """The folder for the output text files."""
        return self._output_folder

    @property
    def currency_symbol(self) -> str:
        """The currency symbol used in the output."""
        return self._currency_symbol

    @property
    def donors_to_display(self) -> str:
        """The number of donors to write out to the output file."""
        return self._donors_to_display

    @property
    def team_goal(self) -> int:
        """The fundraising goal of the team."""
        return self._team_goal

    @property
    def team_captain(self) -> str:
        """The name of the team captain."""
        return self._team_captain

    @property
    def total_raised(self) -> int:
        """The total amount raised by the team."""
        return self._total_raised

    @property
    def num_donations(self) -> int:
        """The number of donations to the team."""
        return self._num_donations

    @property
    def team_avatar_image(self) -> str:
        """The team's avatar image."""
        if self._team_avatar_image:
            return self._team_avatar_image
        else:
            return ""

    @property
    def badge_url(self) -> str:
        """Return the team's badge URL"""
        return self._badge_url

    @property
    def badges(self) -> list[Badge]:
        """Return the list of Team's badges."""
        return self._badges

    @property
    def actvitiy_list(self) -> list[activity.Activity]:
        """Return list of team activities"""
        return self._activity_list

    def _get_team_json(self) -> Tuple:
        """Get team info from JSON api.

        :returns: JSON values for fundraising goal, Captain's name, total value of donations, and the # of donations.
        """
        team_json = donor_drive_comms.get_json(self.team_url)
        if team_json:
            return team_json.get("fundraisingGoal"), team_json.get("captainDisplayName"), \
                   team_json.get("sumDonations"), team_json.get("numDonations"), team_json.get("avatarImageURL")
        team_log.warning("[bold magenta]Could not get team JSON[/bold magenta]")
        return self.team_goal, self.team_captain, self.total_raised, self.num_donations, self.team_avatar_image

    def _update_team_dictionary(self) -> None:
        self._team_info["Team_goal"] = f"{self.currency_symbol}{self.team_goal:,.2f}"
        self._team_info["Team_captain"] = f"{self.team_captain}"
        self._team_info["Team_totalRaised"] = f"{self.currency_symbol}{self.total_raised:,.2f}"
        self._team_info["Team_numDonations"] = f"{self.num_donations}"

    def _get_participants(self, top5: bool) -> List[TeamParticipant]:
        """Get team participant info from api.

        Passes the JSON to the TeamParticipant class for parsing to create a team participant.

        :param top5: If true, get the list sorted by top sum of donations.
        :returns: A list of TeamParticipant objects.
        """
        team_participant_json = donor_drive_comms.get_json(self.team_participant_url, top5)
        if team_participant_json:
            return [TeamParticipant(participant) for participant in team_participant_json]
        team_log.warning("[bold magenta]Couldn't get to URL or possibly no participants.[/bold magenta]")
        if top5:
            return self._top_5_participant_list
        else:
            return self._participant_list

    def _top_participant(self) -> str:
        """Get Top Team Participant.

        This should just grab element 0 from self.top_5_participant_list instead of hitting api twice

        :returns: String formatted information about the top participant.
        """
        if len(self._top_5_participant_list) != 0:
            return (f"{self._top_5_participant_list[0].name} - $"
                    f"{self._top_5_participant_list[0].amount:,.2f}")
        team_log.info("[bold blue] No participants[/bold blue] ")
        return "No participants."

    def _participant_calculations(self) -> None:
        self._participant_calculation_dict['Team_TopParticipantNameAmnt'] = self._top_participant()
        self._participant_calculation_dict['Team_Top5ParticipantsHorizontal'] = \
            donor_drive_comms.multiple_format(self._top_5_participant_list, False, True, self.currency_symbol, 5)
        self._participant_calculation_dict['Team_Top5Participants'] = \
            donor_drive_comms.multiple_format(self._top_5_participant_list, False, False, self.currency_symbol, 5)

    def _update_badges(self) -> None:
        """Add all our badges to the list."""
        self._badges = donor_drive_comms.get_badges(self.badge_url)
        
    def _update_activities(self) -> None:
        """Add activities to the list"""
        self._activity_list = donor_drive_comms.get_activities(self._activity_url)

    def team_run(self) -> None:
        """A public method to update and output team and team participant info."""
        number_of_donations = self.num_donations
        self.team_api_info()
        if self.num_donations > number_of_donations:
            self.participant_run()
            self.donation_run()

    def team_api_info(self) -> None:
        """Get team info from api."""
        self._team_goal, self._team_captain, self._total_raised, self._num_donations,\
            self._team_avatar_image = self._get_team_json()
        self._update_team_dictionary()
        self._update_badges()

    def participant_run(self) -> None:  # pragma: no cover
        """Get and calculate team participant info."""
        self._participant_list = self._get_participants(top5=False)
        self._top_5_participant_list = self._get_participants(top5=True)
        self._participant_calculations()

    def donation_run(self) -> None:  # pragma: no cover
        """Get and calculate donation information."""
        self._donation_list = donor_drive_comms.get_donations(self._donation_list,
                                                                                      self.team_donation_url)
        if self._donation_list:
            self._donation_formatted_output = donor_drive_comms.format_information_for_output(
                self._donation_list, self.currency_symbol, self.donors_to_display, team=True)
            team_avatar_for_html = "<img src=" + self.team_avatar_image + ">"

    def __str__(self):
        team_info = ""
        if self._team_info:
            team_info = f"Team goal is {self._team_info['Team_goal']}."
        if self.team_id:
            return f"A team found at {self.team_url}. {team_info}"
        else:
            return "Not a valid team - no team_id."


if __name__ == "__main__":  # pragma no cover
    # debug next line
    folder = "/home/ermesa/Programming Projects/python/extralife/testOutput"
    my_team = Team("44013", folder, "$", "5")
    my_team.team_api_info()
    my_team.participant_run()



"""Holds all the file and internet input and output."""

import json
import logging
import os
import pathlib
import requests
from rich import print
from rich.logging import RichHandler
from typing import Tuple, Any

import xdgenvpy  # type: ignore

from donordrivepython import api_version_suffix
from donordrivepython.api.donation import Donation
from donordrivepython.api.donor import Donor
from donordrivepython.api.badge import Badge  # type: ignore
from donordrivepython.api import activity # type: ignore

# logging
el_io_log = logging.getLogger("ExtraLife IO")
el_io_log.setLevel(logging.INFO)


def validate_url(url: str):
    el_io_log.debug(f"[bold blue]Checking: {url}[/bold blue]")
    response = requests.get(url)
    el_io_log.debug(f"[bold magenta]Response is: {response.status_code}[/bold magenta]")
    return response.status_code == 200


# JSON/URL
def get_json(url: str, order_by_donations: bool = False, order_by_amount: bool = False) -> dict:
    """Grab JSON from server.

    Connects to server and grabs JSON data from the specified URL. The api server should return JSON with the donation \
    data.

    :param url: api URL for the specific json api point.
    :param order_by_donations: If true, the url param has data appended that will cause the api to return the\
    data in descending order of the sum of donations.
    :param order_by_amount: If true, the url param has data appended that will cause the api to return the\
    data in descending order of the sum of amounts.

    :return: JSON as dictionary with api data.

    :raises: ConnectionError, Timeout
    """
    header = {'User-Agent': 'Extra Life Donation Tracker'}
    response = None
    if order_by_donations and not order_by_amount:
        url += f"?orderBy=sumDonations%20DESC&{api_version_suffix}"
    elif order_by_amount:
        url += f"?orderBy=amount%20DESC&{api_version_suffix}"
    else:
        url += f"?{api_version_suffix}"
    try:
        el_io_log.debug(url)
        response = requests.get(url=url, headers=header)
        return response.json()  # type: ignore
    except requests.exceptions.ConnectionError as this_error:  # pragma: no cover
        el_io_log.error(f"""[bold red]Could not get to {url}.
                Exact error was: {this_error}.
                Check ExtraLifeID. Or server may be unavailable.
                If you can reach that URL from your browser
                and this is not an intermittent problem, please open an issue at:
                https://github.com/djotaku/ELDonationTracker[/bold red]""")
        return {}
    except requests.exceptions.Timeout:  # pragma: no cover
        el_io_log.error(f"[bold red]HTTP code: {response.status_code}[/bold red]")  # type: ignore
        el_io_log.error("[bold red]Timed out while getting JSON. [/bold red]")
        return {}
    except json.decoder.JSONDecodeError:  # pragma: no cover
        el_io_log.error("[bold red]Error with JSON response. [/bold red]")
        el_io_log.error(f"[bold red]The request status code was {response.status_code} from {url=}[/bold red]")
        el_io_log.error(f"[bold red]Message from server was {response.text} from {url=}[/bold red]")
        return {}


def get_donations(donations_or_donors: list, api_url: str, is_donation=True, largest_first=False) -> list:
    """Get the donations from the JSON and create the donation objects.

    If the api can't be reached, the same list is returned. Only new donations are added to the list at the end.

    :param is_donation: True if we are getting data for donations. False if we are getting data for donors.
    :param largest_first: True if we want to sort by largest. False if sort by latest donors or donations.
    :param donations_or_donors: A list consisting of donation.Donation or donor.Donor objects.
    :param api_url: The URL to go to for donations.
    :returns: A list of donation.Donation objects.
    """
    if largest_first and is_donation:
        json_response = get_json(api_url, largest_first, is_donation)
    else:
        json_response = get_json(api_url, largest_first)
    if not json_response:
        el_io_log.error(f"[bold red]Couldn't access JSON endpoint at {api_url}.[/bold red]")
        return donations_or_donors
    else:
        if is_donation:
            donor_or_donation_list = [Donation(this_donation) for this_donation in json_response]
        else:
            donor_or_donation_list = [Donor(this_donor) for this_donor in json_response]  # type: ignore
        if len(donations_or_donors) == 0:  # if I didn't already have donations....
            return donor_or_donation_list
        else:  # add in only the new donations
            for a_donation in reversed(donor_or_donation_list):
                if a_donation not in donations_or_donors:
                    donations_or_donors.insert(0, a_donation)
            return donations_or_donors


def get_badges(api_url: str) -> list[Badge]:
    """Get badges from the api endpoint and create a list to return.

    :param api_url: The URL for the api endpoint.
    :returns: A list of badges.
    """
    json_response = get_json(api_url)
    return [Badge.create_badge(badge_item) for badge_item in json_response]


def get_activities(api_url: str) -> list[activity.Activity]:
    """Get activities from the api endpiont and create a list to return."""
    json_response = get_json(api_url)
    return [activity.create_activity(activity) for activity in json_response]

# File Input and Output
# input
class ParticipantConf:
    """Holds Participant Configuration info.

    :param cls.participant_conf_version: version of participant.conf
    :param cls.version_mismatch: Initialized to False.If true, the user has a different version of the participant.conf.
    :param cls.fields: A dictionary initialed to None for all fields.
    :param self.xdg: By using the PedanticPackage, the directory will be created if it doesn't already exist.
    :param self.participantconf: holds a dictionary of the user's config file.
    """

    participant_conf_version: str = "2.0"
    version_mismatch: bool = False
    fields: dict = {"donor_drive_id": None, "text_folder": None, "currency_symbol": None, "team_id": None,
                    "tracker_image": None, "donation_sound": None, "donors_to_display": None, "font_family": None,
                    "font_size": None, "font_italic": None, "font_bold": None, "font_color": None,
                    "tracker_background_color": None}

    def __init__(self):
        """Load in participant conf and check version."""
        self.xdg = xdgenvpy.XDGPedanticPackage('extralifedonationtracker')
        self.participantconf = self.load_json()
        if self.participantconf['Version'] != self.participant_conf_version:
            el_io_log.warning(f"You are using an old version of participant.conf.\n"
                              f"Your version is: {self.participantconf['Version']}\n"
                              f"Current Version is {self.participant_conf_version}.\n"
                              f"If you are on the commandline, check to see what"
                              f" has changed and add it to your configuration file."
                              f" You will likely have an error later because of this.\n"
                              f"If you are in the GUI, it should prompt you to"
                              f" Migrate or start fresh.")
            self.version_mismatch = True
        self.xdg.XDG_CONFIG_HOME: str
        self.update_fields()

    def load_json(self) -> dict:  # pragma: no cover
        """Load in the config file.

        Checks in a variety of locations for the participant.conf file.\
        First, it checks in the persistent settings location (XDG_CONFIG_HOME)\
        . Then it checks the current directory and one level up. Otherwise it\
        raises an exception and the program stops.

        :return: A dictionary representing the JSON config file.
        :raises: FileNotFoundError
        """

        try:
            el_io_log.info(f"[bold blue]Looking for persistent settings at {self.xdg.XDG_CONFIG_HOME}[/bold blue]")
            with open(f'{self.xdg.XDG_CONFIG_HOME}/participant.conf') as file:
                config = json.load(file)
                file.close()
                el_io_log.info("[bold green]Persistent settings found.[/bold green]")
            return config
        except FileNotFoundError:
            el_io_log.warning("[bold magenta]Persistent settings not found. Checking current directory"
                              f"({os.getcwd()})[/bold magenta]")
        try:
            with open(pathlib.PurePath(__file__).parent.joinpath('.') / 'participant.conf') as file:
                config = json.load(file)
                file.close()
            return config
        except FileNotFoundError:
            el_io_log.warning("[bold magenta]Settings not found in current dir. Checking up one level.[/bold magenta]")
        try:
            with open(pathlib.PurePath(__file__).parent.joinpath('..') / 'participant.conf') as file:
                config = json.load(file)
                file.close()
            return config
        except FileNotFoundError:
            self.get_github_config()
            return self.load_json()

    def get_github_config(self):  # pragma: no cover
        el_io_log.info("[bold blue]Attempting to grab a config file from GitHub.[/bold blue]")
        el_io_log.info(f"[bold blue]Config will be placed at {self.xdg.XDG_CONFIG_HOME}.[/bold blue]")
        url = 'https://github.com/djotaku/ELDonationTracker/raw/master/participant.conf'
        try:
            config_file = requests.get(url)
            open(f"{self.xdg.XDG_CONFIG_HOME}/participant.conf", "wb").write(config_file.content)
        except requests.exceptions.HTTPError:
            el_io_log.error("[bold magenta] Could not find participant.conf on Github. [/bold magenta]"
                            "[bold magenta]Please manually create or download from Github.[/bold magenta]")

    def get_tracker_assets(self, asset: str):  # pragma: no cover
        el_io_log.info(f"[bold blue] Attempting to grab {asset} from Github.[/bold blue] ")
        el_io_log.info(f"[bold blue] {asset} will be placed at the XDG location of: {self.xdg.XDG_DATA_HOME}[/bold "
                       f"blue] ")
        if asset == "image":
            url = 'https://raw.githubusercontent.com/djotaku/ELDonationTracker/master/tracker%20assets/Engineer.png'
        elif asset == "sound":
            url = 'https://raw.githubusercontent.com/djotaku/ELDonationTracker/master/tracker%20assets/Donation.mp3'
        try:
            file = requests.get(url)
            if asset == "image":
                open(f"{self.xdg.XDG_DATA_HOME}/{asset}.png", "wb").write(file.content)
                return f"{self.xdg.XDG_DATA_HOME}/{asset}.png"
            elif asset == "sound":
                open(f"{self.xdg.XDG_DATA_HOME}/{asset}.mp3", "wb").write(file.content)
                return f"{self.xdg.XDG_DATA_HOME}/{asset}.mp3"
            el_io_log.info("[bold blue]file written.[/bold blue]")
        except requests.exceptions.HTTPError:
            el_io_log.error(f"[bold red]Could not get {asset}.[/bold red]")

    def update_fields(self):
        """Update fields variable with data from JSON."""
        for field in self.fields:
            self.fields[field] = self.participantconf.get(f'{field}')
            el_io_log.debug(f"{field}:{self.fields[field]}")

    def write_config(self, config: dict, default: bool):  # pragma: no cover
        """Write config to file.

        Only called from GUI. Commandline user is expected to edit\
        participant.conf manually. Afterward it triggers self.reloadJSON\
        to have the program run with the updated config.

        :param config: A dictionary holding the config values.
        :param default: If True, will save the file in the current directory.
        """
        if default:
            with open('participant.conf', 'w') as outfile:
                json.dump(config, outfile)
        else:
            with open(f'{self.xdg.XDG_CONFIG_HOME}/participant.conf', 'w') as outfile:
                json.dump(config, outfile)
        self.reload_json()

    def get_version(self):
        """Return version."""
        return self.participant_conf_version

    def reload_json(self):
        """Reload JSON and update the fields."""
        self.participantconf = self.load_json()
        self.update_fields()

    def get_cli_values(self) -> Tuple[Any, Any, Any, Any, Any]:
        """Return data required for a CLI-only run.

        :returns: A tuple of strings with config values needed if only\
        running on the commandline.
        """
        return (self.fields["donor_drive_id"], self.fields["text_folder"],
                self.fields["currency_symbol"],
                self.fields["team_id"], self.fields["donors_to_display"])

    def get_text_folder_only(self) -> str:
        """Return text folder data.

        :returns: A string with the text folder location."""
        return self.fields["text_folder"]

    def get_gui_values(self) -> Tuple[Any, Any, Any, Any, Any, Any, Any, Any, Any, Any, Any, Any, Any]:
        """Return values needed for the GUI.

        :returns: A tuple of strings with config values needed if only\
        running the GUI. The two background colors are lists.
        """
        return (self.fields["donor_drive_id"], self.fields["text_folder"],
                self.fields["currency_symbol"], self.fields["team_id"],
                self.fields["tracker_image"], self.fields["donation_sound"],
                self.fields["donors_to_display"], self.fields["font_family"], self.fields["font_size"],
                self.fields["font_italic"], self.fields["font_bold"], self.fields["font_color"],
                self.fields["tracker_background_color"])

    def get_font_info(self):
        """Return values needed to change the font for the tracker.

        :returns: A tuple of strings and a list for the color.
        """
        return (self.fields["font_family"], self.fields["font_size"], self.fields["font_italic"],
                self.fields["font_bold"], self.fields["font_color"])

    def get_tracker_background_color(self):
        """Return value needed to change the tracker background color

        :returns: A list representing the RGB value for the background
        """
        return self.fields["tracker_background_color"]

    def get_if_in_team(self) -> bool:
        """Return True if participant is in a team.

        :returns: True if the participant is in a team.
        """
        el_io_log.debug(self.fields["team_id"])
        return self.fields["team_id"] is not None

    def get_version_mismatch(self) -> bool:
        """Return bool of whether there is a version mismatch.

        :returns: True if the version the user is running is not the same as what this program has as its version.
        """
        return self.version_mismatch

    def get_tracker_image(self) -> str:
        """Return the tracker image location on disk.

        :returns: Location of tracker image on disk.
        """
        return self.fields["tracker_image"]

    def get_tracker_sound(self) -> str:
        """Return the donation sound image location on disk.

        :returns: location of the donation sound on disk.
        """
        return self.fields["donation_sound"]

    def __str__(self):
        return f"A configuration version {self.participant_conf_version} with the following data: {self.fields}"


# Formatting
def single_format(donor, message: bool, currency_symbol: str) -> str:
    """Format string for output to text file.

    This function is for when there is only one donor or donation.
    For example, for the text file holding the most recent donation.

    :param donor: Donor or Donation class object.
    :param message: If True, the message (if any) from the donor object\
    :param currency_symbol: The currency symbol to append to the return string.
    :returns: A string containing the inputs formatted. For example: \n
    John Doe - $100.00 - Thanks for raising money for the kids.
    """
    if message:
        return (f"{donor.name} - {currency_symbol}{donor.amount:.2f}"
                f" - {donor.message}")
    else:
        return f"{donor.name} - {currency_symbol}{donor.amount:.2f}"


def multiple_format(donors, message: bool, horizontal: bool,
                    currency_symbol: str, how_many: int) -> str:
    """Format string for output to text file.

    This function is for when there is are multiple donors or donations.\
    For example, for the text file holding the Top 5 donors.

    :param donors: An iterable of Donors or Donations class objects.
    :param message: If True, the message (if any) from the donor object\
    :param horizontal: If True, format the message horizontally. Else\
    vertically.
    :param currency_symbol: The currency symbol to append to the return string.
    :param how_many: A number for how many donors/donations to append to the\
    string.
    :returns: A string containing the inputs formatted.\n
    Horizontal example: \n
    John Doe - $100.00 - Thanks for raising money for the kids. |\
    Jane Doe - $25.00 - Hurray!\n\n
    Vertical example: \n
    John Doe - $200.00 - a message\n
    Jane Doe - $75.00 - another message
    """
    text = ""
    for donor in range(len(donors)):
        if horizontal:
            text = text + single_format(donors[donor],
                                        message,
                                        currency_symbol) + " | "
        else:
            text = text + single_format(donors[donor],
                                        message,
                                        currency_symbol) + "\n"
        if donor == how_many - 1:
            break
    return text


def format_information_for_output(donation_list: list, currency_symbol: str, donors_to_display: str, team: bool,
                                  donation=True) -> dict:
    """Format the donation attributes for the output files.

    :param donation:
    :param donation_list: A list of donors or donations to format for output.
    :type donation_list: list
    :param currency_symbol: The currency symbol for the output.
    :param donors_to_display: How many donors or donations to display
    :param team: If true, this is creating output for a team. Otherwise, for the participant.
    :returns: A dictionary with the output text formatted correctly.
    """
    donation_formatted_output: dict = {}
    prefix = "Team_" if team else ''
    middle_text = "Donation" if donation else "Donor"
    donation_formatted_output[f'{prefix}Last{middle_text}NameAmnt'] = single_format(donation_list[0],
                                                                                    False, currency_symbol)
    donation_formatted_output[f'{prefix}lastN{middle_text}NameAmts'] = \
        multiple_format(donation_list, False, False, currency_symbol, int(donors_to_display))
    if donation:
        donation_formatted_output[f'{prefix}lastN{middle_text}NameAmtsMessage'] = \
            multiple_format(donation_list, True, False, currency_symbol, int(donors_to_display))
        donation_formatted_output[f'{prefix}lastN{middle_text}NameAmtsMessageHorizontal'] = \
            multiple_format(donation_list, True, True, currency_symbol, int(donors_to_display))
    donation_formatted_output[f'{prefix}lastN{middle_text}NameAmtsHorizontal'] = \
        multiple_format(donation_list, False, True, currency_symbol, int(donors_to_display))
    return donation_formatted_output




"""Participant class to store participant data."""

from dataclasses import dataclass, field
import logging
from rich import print  # type ignore
from rich.logging import RichHandler  # type ignore
import time

from donordrivepython.api import donor, team, donation, badge, activity
from donordrivepython.api import comms as donor_drive_comms

# logging
participant_log = logging.getLogger("Participant")
participant_log.setLevel(logging.INFO)


class Participant:
    """Owns all the attributes under the participant api.

    Also owns the results of any calculated data.

    Donor Drive api api info at https://github.com/DonorDrive/PublicAPI
    """

    def __init__(self, donor_drive_id: str, text_folder: str, currency_symbol: str, team_id: str,
                 donors_to_display: str, base_api_url: str):
        """Load in config from participant.conf and initialize participant variables.
        """
        self._donor_drive_id: str = donor_drive_id
        self._text_folder: str = text_folder
        self._currency_symbol: str = currency_symbol
        self._team_id: str = team_id
        self._donors_to_display: str = donors_to_display
        self._base_api_url: str = base_api_url
        self._participant_url: str = ""
        self._donation_url: str = ""
        self._participant_donor_url: str = ""
        self._my_team: team.Team = None

        # Participant Information
        self._total_raised: int = 0
        self._number_of_donations: int = 0
        self._average_donation: int = 0
        self._goal: int = 0
        self._avatar_image_url: str = ""
        self._event_name: str = ""
        self._donation_link_url: str = ""
        self._stream_url: str = ""
        self._extra_life_page_url: str = ""
        self._created_date_utc: str = ""
        self._stream_is_live: bool = False
        self._sum_pledges: int = 0
        self._team_name: str = ""
        self._is_team_captain: bool = False
        self._display_name: str = ''

        self._participant_formatted_output = {'totalRaised': f"{self.currency_symbol}0.00",
                                              'averageDonation': f"{self.currency_symbol}0.00",
                                              'goal': f"{self.currency_symbol}0.00"}

        # donation information
        self._donation_list: list[donation] = []
        self._ordered_donation_list: list[donation] = []
        self._top_donation = None
        self._top_donation_formatted_output: dict = {'TopDonationNameAmnt': "No Donations Yet"}
        self._donation_formatted_output: dict = {'LastDonationNameAmnt': "No Donations Yet",
                                                 'lastNDonationNameAmts': "No Donations Yet",
                                                 'lastNDonationNameAmtsMessage': "No Donations Yet",
                                                 'lastNDonationNameAmtsMessageHorizontal': "No Donations Yet",
                                                 'lastNDonationNameAmtsHorizontal': "No Donations Yet"}
        # donor information
        self._top_donor = None
        self._top_donor_formatted_output: dict = {'TopDonorNameAmnt': "No Donors Yet"}
        self._donor_list: list[donor] = []
        self._ordered_donor_list: list[donor] = []
        self._donor_formatted_output: dict = {'LastDonorNameAmnt': "No Donations Yet",
                                              'lastNDonorNameAmts': "No Donations Yet",
                                              'lastNDonorNameAmtsMessage': "No Donations Yet",
                                              'lastNDonorNameAmtsMessageHorizontal': "No Donations Yet",
                                              'lastNDonorNameAmtsHorizontal': "No Donations Yet"}

        # other api endpoints
        self._badge_url: str = ''
        self._badges: list[badge.Badge] = []
        self._milestone_url: str = ''
        self._milestones: list[Milestone] = []
        self._incentive_url: str = ''
        self._incentives: list[Incentive] = []
        self._activity_url: str = ''
        self._activities: list[activity.Activity] = []

        # misc
        self._first_run: bool = True
        self._new_donation: bool = False
        self._text_files_exist: bool = False

        self.set_config_values()

    @property
    def donor_drive_id(self) -> str:
        """The participant's extra life ID"""
        return self._donor_drive_id

    @property
    def text_folder(self) -> str:
        """Where the output text files will be written on disk"""
        return self._text_folder

    @property
    def currency_symbol(self) -> str:
        """The currency symbol eg '$' added to the output text files."""
        return self._currency_symbol

    @property
    def team_id(self) -> str:
        """The Team ID if the participant is part of a team."""
        return self._team_id

    @property
    def donors_to_display(self) -> str:
        """For output text files that display multiple donors (or donations), the number of them that
        should be written to the text file.
        """
        return self._donors_to_display

    @property
    def participant_url(self) -> str:
        """api endpoint for the participant"""
        return self._participant_url

    @property
    def donation_url(self) -> str:
        """api endpoint for the Participant's donations."""
        return self._donation_url

    @property
    def participant_donor_url(self) -> str:
        """api Endpoint for the Participant's donors."""
        return self._participant_donor_url

    @property
    def my_team(self) -> team.Team:
        """An instantiation of a team class for the participant's team."""
        return self._my_team

    @property
    def total_raised(self) -> int:
        """Total amount raised by the participant."""
        return self._total_raised

    @property
    def number_of_donations(self) -> int:
        """The number of donations received by the participant."""
        return self._number_of_donations

    @property
    def average_donation(self) -> int:
        """The average amount of donations received by this participant"""
        return self._average_donation

    @property
    def goal(self) -> int:
        """The goal for the amount of the money the participant wishes to raise."""
        return self._goal

    @property
    def avatar_image_url(self) -> str:
        """The user's avatar image on the website."""
        if self._avatar_image_url:
            return self._avatar_image_url
        else:
            return ""

    @property
    def event_name(self) -> str:
        """The event name as obtained from the api."""
        return self._event_name

    @property
    def donation_link_url(self) -> str:
        """The link for making donations to the participant."""
        return self._donation_link_url

    @property
    def stream_url(self) -> str:
        """The URL to the participant's stream on Twitch."""
        return self._stream_url

    @property
    def stream_is_live(self) -> bool:
        """True if the participant is actively streaming."""
        return self._stream_is_live

    @property
    def extra_life_page_url(self) -> str:
        """The URL to the participant's ExtraLife page."""
        return self._extra_life_page_url

    @property
    def created_date_utc(self) -> str:
        """The date the participant created their campaign."""
        return self._created_date_utc

    @property
    def team_name(self) -> str:
        """If the participant is part of a team, this is the team name."""
        return self._team_name

    @property
    def is_team_captain(self) -> bool:
        """True if the participant is the team captain."""
        return self._is_team_captain

    @property
    def sum_pledges(self) -> int:
        """Number of pledges for the participant."""
        return self._sum_pledges

    @property
    def new_donation(self) -> bool:
        """True if the participant receives a new donation."""
        return self._new_donation

    @new_donation.setter
    def new_donation(self, new_donation_status: bool):
        """Change the _new_donation status."""
        self._new_donation = new_donation_status

    @property
    def display_name(self) -> str:
        """Return the participant's display name."""
        return self._display_name

    @property
    def badge_url(self) -> str:
        """Return the participant's badge URL"""
        return self._badge_url

    @property
    def badges(self) -> list[badge.Badge]:  # type: ignore
        """Return the list of participant badges."""
        return self._badges

    @property
    def milestone_url(self) -> str:
        """Return the Milestone URL"""
        return self._milestone_url

    @property
    def milestones(self):
        """Return a list of Milestones"""
        return self._milestones

    @property
    def incentive_url(self) -> str:
        """Return the Incentive URL"""
        return self._incentive_url

    @property
    def incentives(self):
        """Return a list of Incentives"""
        return self._incentives

    @property
    def activities(self):
        """Return a list of Activities"""
        return self._activities

    def set_config_values(self) -> None:
        """Set participant values, create URLs, and create Team."""
        # urls
        self._participant_url = f"{self._base_api_url}/participants/{self.donor_drive_id}"
        self._donation_url = f"{self.participant_url}/donations"
        self._participant_donor_url = f"{self.participant_url}/donors"
        self._badge_url = f"{self.participant_url}/badges"
        self._milestone_url = f"{self.participant_url}/milestones"
        self._incentive_url = f"{self.participant_url}/incentives"
        self._activity_url = f"{self.participant_url}/activity"

        if self.team_id:
            self._my_team = team.Team(self.team_id, self.text_folder, self.currency_symbol, self.donors_to_display,
                                      self._base_api_url)

    def _get_participant_info(self):
        """Get JSON data for participant information.

        :returns: JSON data for self.total_raised, self.number_of_donations, and self.goal.
        """
        participant_json = donor_drive_comms.get_json(self.participant_url)
        if not participant_json:
            participant_log.warning("[bold red]Couldn't access participant JSON.[/bold red]")
            return self.total_raised, self.number_of_donations, self.goal, self.avatar_image_url, self.event_name, \
                self.donation_link_url, self.stream_url, self.extra_life_page_url, self.created_date_utc,\
                self.stream_is_live, self.sum_pledges, self.team_name, self.is_team_captain, self.display_name
        else:
            if self.my_team:
                team_name = participant_json.get('teamName')
                is_team_captain = participant_json.get('isTeamCaptain')
            else:
                team_name = ''
                is_team_captain = False
            return participant_json.get('sumDonations'), participant_json.get('numDonations'), \
                participant_json.get('fundraisingGoal'), participant_json.get('avatarImageURL'), \
                participant_json.get('eventName'), participant_json.get('links').get("donate"), \
                participant_json.get('links').get('stream'), participant_json.get('links').get('page'), \
                participant_json.get('createdDateUTC'), participant_json.get("streamIsLive"), \
                participant_json.get('sumPledges'), team_name, is_team_captain, participant_json.get('displayName')

    def _format_participant_info_for_output(self, participant_attribute) -> str:
        """Format participant info for output to text files.

        :param participant_attribute: the data to be formatted for the output.
        :returns: A string with the formatted information.
        """
        return f"{self.currency_symbol}{participant_attribute:,.2f}"

    def _fill_participant_dictionary(self) -> None:
        """Fill up self.participant_formatted_output ."""
        self._participant_formatted_output["totalRaised"] = \
            self._format_participant_info_for_output(self.total_raised)
        self._participant_formatted_output["averageDonation"] = \
            self._format_participant_info_for_output(self.average_donation)
        self._participant_formatted_output["goal"] = self._format_participant_info_for_output(self.goal)
        self._participant_formatted_output["numDonations"] = str(self.number_of_donations)

    def _calculate_average_donation(self):
        """Calculate the average donation amount.

        :returns: The average or 0.
        """
        try:
            return self.total_raised / self.number_of_donations
        except ZeroDivisionError:
            return 0

    def _get_top_donations(self):  # pragma: no cover
        """Return top donations from server.

        Uses donor drive's sorting to get the top donation."""
        return donor_drive_comms.get_donations(self._ordered_donation_list, self.donation_url, True, True)

    def _get_top_donors(self):  # pragma: no cover
        """Return Top Donors from server.

        Uses donor drive's sorting to get the top guy or gal.
        """
        return donor_drive_comms.get_donations(self._ordered_donor_list, self.participant_donor_url, False, True)

    def _get_donors(self):  # pragma: no cover
        """Return Donors from server."""
        return donor_drive_comms.get_donations(self._ordered_donor_list, self.participant_donor_url, False, False)

    def _format_donor_information_for_output(self) -> None:
        """Format the donor attributes for the output files."""
        self._top_donor_formatted_output['TopDonorNameAmnt'] = donor_drive_comms.single_format(self._top_donor, False,
                                                                                               self.currency_symbol)
        self._donor_formatted_output = donor_drive_comms.format_information_for_output(
            self._donor_list, self.currency_symbol, self.donors_to_display, team=False, donation=False)

    def _format_donation_information_for_output(self) -> None:
        """Format the donation attributes for the output files."""
        self._donation_formatted_output = donor_drive_comms.format_information_for_output(
            self._donation_list, self.currency_symbol, self.donors_to_display, team=False)
        self._top_donation_formatted_output['TopDonationNameAmnt'] = donor_drive_comms.single_format(self._top_donation,
                                                                                                     False,
                                                                                                     self.currency_symbol)

    def update_participant_attributes(self) -> None:  # pragma: no cover
        """Update participant attributes.

         A public method that will update the Participant object with data from self.participant_url.

         Also called from the main loop.
         """
        self._total_raised, self._number_of_donations, self._goal, self._avatar_image_url, \
            self._event_name, self._donation_link_url, self._stream_url, \
            self._extra_life_page_url, self._created_date_utc, self._stream_is_live, \
            self._sum_pledges, self._team_name, self._is_team_captain, self._display_name = self._get_participant_info()
        self._average_donation = self._calculate_average_donation()

    def update_donation_data(self) -> None:
        """Update donation data."""
        if self.number_of_donations > 0:
            self._donation_list = donor_drive_comms.get_donations(self._donation_list, self.donation_url)
            self._ordered_donation_list = self._get_top_donations()
            try:
                self._top_donation = self._ordered_donation_list[0]
            except IndexError:  # pragma: no cover
                pass

    def update_donor_data(self) -> None:
        """Update donor data."""
        if self.number_of_donations > 0:
            self._donor_list = self._get_donors()
            # anonymous donors mess things up because they don't populate the donor api endpoint.
            # So this check prevents a crash.
            if self._donor_list:
                self._ordered_donor_list = self._get_top_donors()
                self._top_donor = self._ordered_donor_list[0]

    def _update_badges(self) -> None:
        """Add all our badges to the list."""
        self._badges = donor_drive_comms.get_badges(self.badge_url)

    def _update_milestones(self) -> None:
        """Add all milestones to the list"""
        json_response = donor_drive_comms.get_json(self.milestone_url)
        self._milestones = [Milestone.create_milestone(milestone_item) for milestone_item in json_response]

    def _update_incentives(self) -> None:
        """Add all incentives to list"""
        json_response = donor_drive_comms.get_json(self.incentive_url)
        self._incentives = [Incentive.create_incentive(incentive) for incentive in json_response]

    def _update_activities(self) -> None:
        """Add Participant Activities to list"""
        json_response = donor_drive_comms.get_json(self._activity_url)
        self._activities = [activity.create_activity(item) for item in json_response]

    def run(self) -> None:
        """Run to get participant, donation, donor, and team data and output to text files."""
        number_of_donations = self.number_of_donations
        self.update_participant_attributes()
        self._update_incentives()
        self._update_activities()
        # Below is protection against a situation where the api is unavailable.
        # Prevents bad data being written to the participant output. Based on the assumption that it would
        # absurd to have a goal of $0.
        if self._first_run or self.number_of_donations > number_of_donations:
            if not self._first_run:
                print("[bold green]A new donation![/bold green]")
                self._new_donation = True
            self.update_donation_data()
            self.update_donor_data()
            self._update_badges()
            self._update_milestones()
        # TEAM BLOCK ############################################
        if self.team_id:
            self.my_team.team_run()
        ##########################################################
        self._first_run = False
        participant_log.info("Finished checking api and updating text files!")

    def __str__(self):
        if self.my_team:
            return f"A participant with Extra Life ID {self.donor_drive_id}. Team info: {self.my_team}"
        else:
            return f"A participant with Extra Life ID {self.donor_drive_id}."


@dataclass
class Milestone:  # type: ignore
    """Fundraiser milestones associated with a Participant.

    May not be available for all instances of Donor Drive.

    More information at: https://github.com/DonorDrive/PublicAPI/blob/master/resources/milestones.md
    """
    description: str
    fundraising_goal: float
    is_active: bool
    milestone_id: str
    is_complete: bool = False
    links: dict = field(default_factory=dict)
    end_date_utc: str = ""
    start_date_utc: str = ""

    @staticmethod
    def create_milestone(json_data: dict):    # type: ignore
        """Uses the provided JSON data to create a Milestone object."""
        # let's start with the data guaranteed to be returned
        description = json_data.get("description")
        fundraising_goal = json_data.get('fundraisingGoal')
        is_active = json_data.get('isActive')
        milestone_id = json_data.get('milestoneID')
        # now the ones that may not be there
        is_complete = json_data.get('isComplete', False)
        links = json_data.get('links', {})
        end_date_utc = json_data.get('endDateUTC', '')
        start_date_utc = json_data.get('startDateUTC', '')
        return Milestone(description, fundraising_goal, is_active, milestone_id, is_complete, links,  # type: ignore
                         end_date_utc, start_date_utc)  # type: ignore


@dataclass
class Incentive:  # type: ignore
    """Fundraiser incentives associated with a Participant.

    May not be available for all instances of Donor Drive.

    More information at:https://github.com/DonorDrive/PublicAPI/blob/master/resources/incentives.md
    """
    amount: float
    description: str
    incentive_id: str
    is_active: str
    end_date_utc: str = ''
    incentive_image_url: str = ''
    links: dict = field(default_factory=dict)
    start_date_utc: str = ''
    quantity: int = 0
    quantity_claimed: int = 0

    @staticmethod
    def create_incentive(json_data: dict):
        """Uses the provided JSON data to create an Incentive object."""
        # guaranteed data from the api endpoint
        amount = json_data.get("amount")
        description = json_data.get("description")
        incentive_id = json_data.get('incentiveID')
        is_active = json_data.get("isActive")
        # optional data
        end_data_utc = json_data.get("endDateUTC", '')
        incentive_image_url = json_data.get("incentiveImageURL", '')
        links = json_data.get("links", {})
        start_date_utc = json_data.get("startDateUTC", '')
        quantity = json_data.get("quantity", 0)
        quantity_claimed = json_data.get("quantityClaimed", 0)
        return Incentive(amount, description, incentive_id, is_active, end_data_utc,  # type: ignore
                         incentive_image_url, links, start_date_utc, quantity, quantity_claimed)  # type: ignore


if __name__ == "__main__":  # pragma: no cover
    participant_conf = donor_drive_comms.ParticipantConf()
    print(participant_conf)
    (extralife_id, text_folder, currency_symbol, team_id, donors_to_display) = participant_conf.get_cli_values()
    p = Participant("478153", text_folder, currency_symbol, team_id, donors_to_display,
                    "https://www.extra-life.org/api")
    print(p)
    while True:
        p.run()
        for an_activity in p.activities:
            print(f"{an_activity}\n")
        time.sleep(15)



"""Test methods in donor class."""

from eldonationtracker.api import donor


def test_donor_attributes_none_missing():
    donor1_json = {"displayName": "donor1", "sumDonations": "45", "donorID": 1000111,
                   'avatarImageURL': "http://someplace.com/image.jpg", "numDonations": 2}
    donor1 = donor.Donor(donor1_json)
    assert donor1.name == "donor1"
    assert donor1.donor_id == 1000111
    assert donor1.image_url == "http://someplace.com/image.jpg"
    assert donor1.amount == 45
    assert donor1.number_of_donations == 2


def test_donor_attributes_name_missing():
    donor1_json = {"sumDonations": "45", "donorID": 1000111,
                   'avatarImageURL': "http://someplace.com/image.jpg", "numDonations": 2}
    donor1 = donor.Donor(donor1_json)
    assert donor1.name == "Anonymous"
    assert donor1.donor_id == 1000111
    assert donor1.image_url == "http://someplace.com/image.jpg"
    assert donor1.amount == 45
    assert donor1.number_of_donations == 2


def test_donor_lt_whole_numbers():
    """Test to make sure comparison works.

    Uses whole dollar amounts.
    """
    donor1_json = {"displayName": "donor1", "sumDonations": "45"}
    donor2_json = {"displayName": "donor2", "sumDonations": "30"}
    donor1 = donor.Donor(donor1_json)
    donor2 = donor.Donor(donor2_json)
    assert donor2 < donor1


def test_donor_lt_change():
    """Test to make sure comparison works.

    Uses dollars and cents.
    """
    donor1_json = {"displayName": "donor1", "sumDonations": "30.00"}
    donor2_json = {"displayName": "donor2", "sumDonations": "30.01"}
    donor1 = donor.Donor(donor1_json)
    donor2 = donor.Donor(donor2_json)
    assert donor1 < donor2


def test_donors_equal():
    """Test to make sure comparison works.

    This time they're equal.
    """
    donor1_json = {"displayName": "donor1", "sumDonations": "30"}
    donor2_json = {"displayName": "donor2", "sumDonations": "30"}
    donor1 = donor.Donor(donor1_json)
    donor2 = donor.Donor(donor2_json)
    assert donor1 == donor2


def test_str():
    donor1_json = {"displayName": "donor1", "sumDonations": "45", "donorID": 1000111,
                   'avatarImageURL': "http://someplace.com/image.jpg", "numDonations": 2}
    donor1 = donor.Donor(donor1_json)
    assert str(donor1) == "A donor named donor1 with donor ID 1000111 who has donated $45.00 over 2 donations."


def test_emoji_in_name():
    """Test to make sure that emoji in the donor's name doesn't crash the program."""
    emoji_donor_json = {"displayName": "🍀🍀🍀🍀", "sumDonations": "45", "donorID": 1000111,
                   'avatarImageURL': "http://someplace.com/image.jpg", "numDonations": 2}
    emoji_donor = donor.Donor(emoji_donor_json)
    assert emoji_donor.name == "🍀🍀🍀🍀"



"""A class to hold the Donation attributes and methods."""


class Donation:
    """Donation Attributes.

    Class exists to provide attributes for a donation based on what comes in from the JSON so that it doesn't have to be
     traversed each time a donor action needs to be taken.
    """

    def __init__(self, json):
        """Load in values from class initialization.

        :param json: JSON attributes from the api
        :type json: json
        """
        self._name, self._message, self._amount, self._donor_id, self._avatar_url, self._donation_date,\
            self._donation_id = self.json_to_attributes(json)

    @property
    def name(self) -> str:
        """the name of the donor for this donation. If the donor wished to stay anonymous, the variable is set to/
     'Anonymous'"""
        return self._name

    @property
    def message(self) -> str:
        """the message associated with the donation."""
        return self._message

    @property
    def amount(self) -> int:
        """the amount of the donation. If they blocked it from showing it is set to 0."""
        return self._amount

    @property
    def donor_id(self) -> str:
        """the donor drive ID of the donor who made this donation"""
        return self._donor_id

    @property
    def avatar_url(self) -> str:
        """the URL of the avatar associated with this donation."""
        return self._avatar_url

    @property
    def donation_date(self) -> str:
        """the date of the donation."""
        return self._donation_date

    @property
    def donation_id(self) -> str:
        """The donor drive ID of this donation."""
        return self._donation_id

    @staticmethod
    def json_to_attributes(json):
        """Convert api JSON values to Donation attributes.

        :param json: JSON attributes from the api
        :type json: json
        """
        if json.get('displayName') is not None:
            name = json.get('displayName')
        else:
            name = "Anonymous"
        message = json.get('message')
        amount = json.get('amount') if json.get('amount') is not None else 0
        donor_id = json.get('donorID')
        avatar_url = json.get('avatarImageURL')
        donation_date = json.get('createdDateUTC')
        donation_id = json.get('donationID')
        return name, message, amount, donor_id, avatar_url, donation_date, donation_id

    def __eq__(self, other):
        """Donation equal comparison.

        :returns: True if this donation has the same donation ID.
        """
        return self.donation_id == other.donation_id

    def __str__(self):
        return f"A donation by {self.name} in the amount of ${self.amount} with the message '{self.message}'"



from datetime import datetime
from tzlocal import get_localzone

# type: ignore

"""Activities from the Activity Endpoint."""


class Activity:
    """Activity class"""

    def __init__(self, created_date, image_url, type):
        self.created_date: str = created_date
        self.image_url: str = image_url
        self.type: str = type

    def better_date(self):
        """Convert the date to a prettier format."""
        my_time_zone = get_localzone()
        date_and_time_of_activity_utc = datetime.strptime(self.created_date, '%Y-%m-%dT%H:%M:%S.%f%z')
        date_and_time_of_activity_my_time_zone = date_and_time_of_activity_utc.astimezone(my_time_zone)
        pretty_date_time = date_and_time_of_activity_my_time_zone.strftime('%x - %X')
        return pretty_date_time
    def __str__(self):
        return f"{self.better_date()} - An activity of type {self.type} created "


class DonationActivity(Activity):
    """A donation activity"""

    def __init__(self, amount, created_date, image_url, is_incentive, message, title, type):
        super().__init__(created_date, image_url, type)
        self.amount: float = amount
        self.is_incentive: Bool = is_incentive
        self.message: str = message
        self.title: str = title

    def __str__(self):
        if self.is_incentive:
            return f"{self.better_date()} - Incentive reached: {self.message} with {self.title} donation of ${self.amount}."
        return f"{self.better_date()}- {self.title} donation in the amount of ${self.amount}."


class BadgeActivity(Activity):
    """"Badge Activity"""

    def __init__(self, created_date, image_url, message, title, type):
        super().__init__(created_date, image_url, type)
        self.message: str = message
        self.title: str = title

    def __str__(self):
        return f"{self.better_date()}- {self.message}: '{self.title}' badge earned!!"


def create_activity(json_data: dict):
    """
    To deal with the activity endpoint, this will create activities.
    """
    amount = json_data.get('amount')
    created_date = json_data.get('createdDateUTC')
    image_url = json_data.get('imageURL')
    is_incentive = json_data.get('isIncentive')
    message = json_data.get('message')
    title = json_data.get('title')
    type = json_data.get("type")
    if type == "donation":
        return DonationActivity(amount, created_date, image_url, is_incentive, message, title, type)
    elif type == "participantBadge" or "teamBadge":
        return BadgeActivity(created_date, image_url, message , title, type,)
    else:
        return Activity(created_date, image_url, type)



# type: ignore

import json
from unittest import mock

from eldonationtracker.api.participant import Participant, Milestone
import eldonationtracker.api.participant
from eldonationtracker.api.badge import Badge

config = ("12345", "textfolder", "$", "45678", "5")
fake_participant_conf = mock.Mock()
fake_participant_conf.get_cli_values.return_value = config

config_no_team = ("12345", "textfolder", "$", None, "5")
fake_participant_conf_no_team = mock.Mock()
fake_participant_conf_no_team.get_cli_values.return_value = config_no_team

fake_participant_info = {"displayName": "Eric Mesa", "fundraisingGoal": 600.00, "participantID": 449263.0,
                         "eventName": "Extra Life 2021", "teamName": "Twitchclub", "isTeamCaptain": False,
                         "avatarImageURL": "//assets.donordrive.com/extralife/images/$avatars$/constituent_D4DC394A-C293-34EB-4162ECD2B8BF4C17.jpg",
                         "streamIsLive": False,
                         "links":
                             {"donate":
                                  "https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=449263#donate",
                              "stream": "https://player.twitch.tv/?channel=djotaku",
                              "page":
                                  "https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=449263"},
                         "createdDateUTC": "2021-01-03T21:45:29.523+0000", "eventID": 550.0, "sumDonations": 75.00,
                         "sumPledges": 0.00, "numDonations": 2.0}
fake_participant_info_no_team = {"displayName": "Eric Mesa", "fundraisingGoal": 600.00, "participantID": 449263.0,
                                 "eventName": "Extra Life 2021",
                                 "avatarImageURL": "//assets.donordrive.com/extralife/images/$avatars$/constituent_D4DC394A-C293-34EB-4162ECD2B8BF4C17.jpg",
                                 "streamIsLive": False,
                                 "links":
                                 {"donate":
                                  "https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=449263#donate",
                                  "stream": "https://player.twitch.tv/?channel=djotaku",
                                  "page":
                                   "https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=449263"},
                                 "createdDateUTC": "2021-01-03T21:45:29.523+0000", "eventID": 550.0,
                                 "sumDonations": 75.00, "sumPledges": 0.00, "numDonations": 2.0}

fake_donations = {"displayName": "Sean Gibson", "participantID": 401280, "amount": 25.00, "donorID": "54483486D840B7EA",
                  "avatarImageURL": "//assets.donordrive.com/clients/extralife/img/avatar-constituent-default.gif",
                  "createdDateUTC": "2020-02-11T17:22:23.963+0000", "eventID": 547, "teamID": 50394,
                  "donationID": "861A3C59D235B4DA"}, {"displayName": "Eric Mesa", "participantID": 401280,
                                                      "amount": 25.00, "donorID": "4162ECD2B8BF4C17",
                                                      "avatarImageURL": "//assets.donordrive.com/extralife/images/$avat"
                                                                        "ars$/constituent_D4DC394A-C293-34EB-4162ECD2B"
                                                                        "8BF4C17.jpg",
                                                      "createdDateUTC": "2020-01-05T20:35:28.897+0000",
                                                      "eventID": 547, "teamID": 50394, "donationID": "7D430E9E9AF79686"}

donor1_json = {"displayName": "donor1", "sumDonations": "45", "donorID": 1000111,
               'avatarImageURL': "http://someplace.com/image.jpg", "numDonations": 2}
donor2_json = {"displayName": "donor2", "sumDonations": "20", "donorID": 1000112,
               'avatarImageURL': "http://someplace.com/image.jpg", "numDonations": 3}
donor1 = eldonationtracker.api.donor.Donor(donor1_json)
donor2 = eldonationtracker.api.donor.Donor(donor2_json)
fake_top_donor_json = [{"displayName": "Top Donor", "sumDonations": "100", "donorID": 1000111,
                       'avatarImageURL': "http://someplace.com/image.jpg", "numDonations": 2}]

donation1_json = {"displayName": "Donor 1", "participantID": '4939d', "amount": 34.51, "donorID": "FAKE3C59D235B4DA",
                  "avatarImageURL": "//image.png", "message": "Good job! From Donor 1",
                  "createdDateUTC": "2020-02-11T17:22:23.963+0000", "eventID": 547, "teamID": 50394,
                  "donationID": "861A3C59D235B4DA"}
donation2_json = {"displayName": "Donor 2", "participantID": '4939d', "amount": 34.51, "donorID": "FAKE3C59D235B4DA",
                  "avatarImageURL": "//image.png",
                  "createdDateUTC": "2020-02-11T17:22:23.963+0000", "eventID": 547, "teamID": 50394,
                  "donationID": "861A3C59D235B4DB"}
donation_anonymous_json = {"displayName": None, "participantID": '4939d', "amount": 34.51,
                           "donorID": "FAKE3C59D235B4DA", "avatarImageURL": "//image.png",
                           "message": "Good job! From Donor 3",
                           "createdDateUTC": "2020-02-11T17:22:23.963+0000", "eventID": 547, "teamID": 50394,
                           "donationID": "861A3C59D235B4DC"}


fake_extralife_io = mock.Mock()
fake_extralife_io.get_json.return_value = fake_participant_info
fake_extralife_io.get_JSON_donations.return_value = fake_donations
fake_extralife_io.get_JSON_no_json.return_value = {}
fake_extralife_io.get_JSON_donations_no_json.return_value = {}
fake_extralife_io.get_JSON_top_donor_no_json.return_value = {}
fake_extralife_io.get_json_no_team.return_value = fake_participant_info_no_team

magic_fake_extralife_io = mock.MagicMock()
magic_fake_extralife_io.get_JSON_top_donor.return_value = fake_top_donor_json

fake_participant = mock.Mock()
fake_participant.write_text_files.return_value = None
fake_participant._format_donation_information_for_output.return_value = None
fake_participant._format_donor_information_for_output.return_value = None

fake_participant_for_run = mock.Mock()
fake_participant_for_run.update_participant_attributes.return_value = None
fake_participant_for_run.output_participant_data.return_value = None
fake_participant_for_run.write_ipc.return_value = None
fake_participant_for_run.update_donation_data.return_value = None
fake_participant_for_run.output_donation_data.return_value = None
fake_participant_for_run.update_donor_data.return_value = None
fake_participant_for_run.output_donor_data.return_value = None
fake_participant_for_run._update_milestones.return_value = None
fake_participant_for_run.output_milestone_data.return_value = None
fake_participant_for_run.my_team.team_run.return_value = None
fake_participant_for_run.my_team.participant_run.return_value = None
fake_participant_for_run._update_incentives.return_value = None
fake_participant_for_run.output_incentive_data.return_value = None


def test_api_variables():
    """Test that api variables are properly assigned."""
    my_participant = Participant(fake_participant_conf)
    assert (my_participant.donor_drive_id, my_participant.text_folder,
            my_participant.currency_symbol, my_participant.team_id,
            my_participant.donors_to_display) == ("12345", "textfolder", "$", "45678", "5")


def test_urls():
    """Make sure the api URLs are properly constructed."""
    my_participant = Participant(fake_participant_conf)
    assert my_participant.participant_url == "https://www.extra-life.org/api/participants/12345"
    assert my_participant.donation_url == "https://www.extra-life.org/api/participants/12345/donations"
    assert my_participant.participant_donor_url == "https://www.extra-life.org/api/participants/12345/donors"


def test_str_with_a_team():
    """Test the str(participant) string if the participant is part of a team."""
    my_participant = Participant(fake_participant_conf)
    assert str(my_participant) == "A participant with Extra Life ID 12345. Team info: A team found at https://www.extra-life.org/api/teams/45678. "


def test_str_without_a_team():
    """Test the str(participant) string if the participant is not part of a team."""
    my_participant = Participant(fake_participant_conf_no_team)
    assert str(my_participant) == "A participant with Extra Life ID 12345."


def test_new_donation_property():
    """Test that new donation property and setter are working correctly."""
    my_participant = Participant(fake_participant_conf_no_team)
    assert my_participant.new_donation is False
    my_participant.new_donation = True
    assert my_participant.new_donation


@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "get_json", fake_extralife_io.get_json)
def test_get_participant_info():
    """Make sure the api info for the participant is properly assigned."""
    my_participant = Participant(fake_participant_conf)
    assert my_participant._get_participant_info() == (75.0, 2, 600, "//assets.donordrive.com/extralife/images/$avatars$/constituent_D4DC394A-C293-34EB-4162ECD2B8BF4C17.jpg",
                                                      'Extra Life 2021', 'https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=449263#donate',
                                                      'https://player.twitch.tv/?channel=djotaku',
                                                      'https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=449263',
                                                      '2021-01-03T21:45:29.523+0000', False, 0, 'Twitchclub', False,
                                                      'Eric Mesa')


@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "get_json", fake_extralife_io.get_json_no_team)
def test_get_participant_info_no_team():
    """Make sure the api info for the participant is properly assigned."""
    my_participant = Participant(fake_participant_conf)
    my_participant._my_team = None
    assert my_participant._get_participant_info() == (75.0, 2, 600, "//assets.donordrive.com/extralife/images/$avatars$/constituent_D4DC394A-C293-34EB-4162ECD2B8BF4C17.jpg",
                                                      'Extra Life 2021', 'https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=449263#donate',
                                                      'https://player.twitch.tv/?channel=djotaku',
                                                      'https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=449263',
                                                      '2021-01-03T21:45:29.523+0000', False, 0, '', False,
                                                      'Eric Mesa')


@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "get_json", fake_extralife_io.get_JSON_no_json)
def test_get_participant_info_no_json():
    """Ensure that the proper values are returned if the JSON values are not retrieved from the api."""
    my_participant = Participant(fake_participant_conf)
    assert my_participant._get_participant_info() == (0, 0, 0, '', '', '', '', '', '', False, 0, '', False, '')


def test_format_participant_info_for_output():
    """Make sure the right values are grabbed from the participant properties."""
    my_participant = Participant(fake_participant_conf)
    my_participant._total_raised = 400
    my_participant._average_donation = 100
    my_participant._goal = 1000
    assert my_participant._format_participant_info_for_output(my_participant.total_raised) == "$400.00"
    assert my_participant._format_participant_info_for_output(my_participant.average_donation) == "$100.00"
    assert my_participant._format_participant_info_for_output(my_participant.goal) == "$1,000.00"


def test_fill_participant_dictionary():
    """Make sure the output formatted dictionary has the right info."""
    my_participant = Participant(fake_participant_conf)
    my_participant._total_raised = 400
    my_participant._average_donation = 100
    my_participant._goal = 1000
    my_participant._fill_participant_dictionary()
    assert my_participant._participant_formatted_output["totalRaised"] == "$400.00"
    assert my_participant._participant_formatted_output["averageDonation"] == "$100.00"
    assert my_participant._participant_formatted_output["goal"] == "$1,000.00"


def test_calculate_average_donation():
    """Make sure the average donation is properly calculated."""
    my_participant = Participant(fake_participant_conf)
    my_participant._total_raised = 100
    my_participant._number_of_donations = 2
    assert my_participant._calculate_average_donation() == 50


def test_calculate_average_donation_no_donations():
    """Make sure the average donation is properly calculated if there haven't been any donations yet."""
    my_participant = Participant(fake_participant_conf)
    my_participant._total_raised = 0
    my_participant._number_of_donations = 0
    assert my_participant._calculate_average_donation() == 0


@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "get_json", fake_extralife_io.get_JSON_top_donor_no_json)
def test_get_top_donor_no_json():
    """Make sure the top donor works correctly if the JSON was not returned."""
    my_participant = Participant(fake_participant_conf)
    my_participant._top_donor = donor1
    my_participant.update_donor_data()
    assert my_participant._top_donor == donor1


def test_format_donor_information_for_output():
    """Make sure the donor information is properly formatted for the users."""
    my_participant = Participant(fake_participant_conf)
    my_participant._top_donor = donor1
    my_participant._donor_list = [donor2, donor1]
    my_participant._format_donor_information_for_output()
    assert my_participant._top_donor_formatted_output['TopDonorNameAmnt'] == "donor1 - $45.00"
    assert my_participant._donor_formatted_output['LastDonorNameAmnt'] == "donor2 - $20.00"
    assert my_participant._donor_formatted_output['lastNDonorNameAmts'] == "donor2 - $20.00\ndonor1 - $45.00\n"


def test_format_donation_information_for_output():
    """Test donation output for the users."""
    my_participant = Participant(fake_participant_conf)
    donation1 = eldonationtracker.api.donation.Donation(donation1_json)
    donation2 = eldonationtracker.api.donation.Donation(donation2_json)
    donation3 = eldonationtracker.api.donation.Donation(donation_anonymous_json)
    my_participant._donation_list = [donation3, donation2, donation1]
    my_participant._top_donation = donation1
    my_participant._format_donation_information_for_output()
    assert my_participant._donation_formatted_output['LastDonationNameAmnt'] == "Anonymous - $34.51"
    assert my_participant._donation_formatted_output['lastNDonationNameAmts'] == "Anonymous - $34.51\nDonor 2 - $34.51" \
                                                                                "\nDonor 1 - $34.51\n"
    assert my_participant._donation_formatted_output['lastNDonationNameAmtsMessage'] == "Anonymous - $34.51 - Good job" \
                                                                                       "! From Donor 3\nDonor 2 - $34" \
                                                                                       ".51 - None\nDonor 1 - $34.51" \
                                                                                       " - Good job! From Donor 1\n"
    assert my_participant._donation_formatted_output['lastNDonationNameAmtsMessageHorizontal'] == "Anonymous - $34.51" \
                                                                                                 " - Good job! From " \
                                                                                                 "Donor 3 | Donor 2 -" \
                                                                                                 " $34.51 - None | " \
                                                                                                 "Donor 1 - $34.51 - " \
                                                                                                 "Good job! From " \
                                                                                                 "Donor 1 | "
    assert my_participant._donation_formatted_output['lastNDonationNameAmtsHorizontal'] == "Anonymous - $34.51" \
                                                                                          " | Donor 2 - $34.51" \
                                                                                          " | Donor 1 - $34.51 | "


def test_update_donation_data_no_donations():
    """Make sure the donation data is updated correctly if there are no donations."""
    my_participant = Participant(fake_participant_conf)
    my_participant.update_donation_data()
    assert my_participant._donation_list == []


@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "get_json", fake_extralife_io.get_JSON_donations)
def test_update_donation_data_preexisting_donations():
    my_participant = Participant(fake_participant_conf)
    my_participant._number_of_donations = 2
    my_participant.update_donation_data()
    assert my_participant._donation_list[0].name == "Sean Gibson"


def test_update_donor_data_no_donations():
    my_participant = Participant(fake_participant_conf)
    my_participant.update_donor_data()
    assert my_participant._top_donor is None


magic_fake_extralife_io_donor = mock.MagicMock()
magic_fake_extralife_io_donor.get_donors.return_value = [donor1]


@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "get_donations", magic_fake_extralife_io_donor.get_donors)
def test_update_donor_data():
    my_participant = Participant(fake_participant_conf)
    my_participant._number_of_donations = 2
    my_participant.update_donor_data()
    assert my_participant._top_donor.name == "donor1"


example_json = """[
    {
        "description": "Raised 100 dollars!",
        "title": "100 Club Badge",
        "unlockedDateUTC": "2019-10-30T18:01:23.430+0000",
        "badgeImageURL": "http://assets.donordrive.com/try/images/$event508$/badge_2F7819D3_C019_3C7D_B9D716687CEEC0A5.png",
        "badgeCode": "100-club-badge"
      },
      {
        "description": "Sent 25 donation invite emails!",
        "title": "Enthusiastic Participant Badge",
        "unlockedDateUTC": "2019-09-18T15:47:39.107+0000",
        "badgeImageURL": "https://assets.donordrive.com/try/images/$event508$/badge_DCB0A883_BC0A_97DB_639B4D7BFDEC638E.png",
        "badgeCode": "enthusiastic-participant-badge"
      }
    ]"""
example_json_as_json = json.loads(example_json)
test_badges = [Badge.create_badge(a_badge) for a_badge in example_json_as_json]
magic_fake_badges = mock.MagicMock()
magic_fake_badges.get_badges.return_value = test_badges


@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "get_badges", magic_fake_badges.get_badges)
def test_update_badges():
    """Test to make sure that badges are updated."""
    my_participant = Participant(fake_participant_conf)
    assert my_participant.badges == []
    my_participant._update_badges()
    assert my_participant.badges[0].badge_code == "100-club-badge"


magic_fake_milestones = mock.MagicMock()
magic_fake_milestones.get_milestones.return_value = "something"

# note: order matters - this one needs to go before the one where _format_donation...output is called or the not called
# fails.
@mock.patch.object(eldonationtracker.api.participant.Participant, 'write_text_files', fake_participant.write_text_files)
@mock.patch.object(eldonationtracker.api.participant.Participant, '_format_donation_information_for_output',
                   fake_participant._format_donation_information_for_output)
def test_output_donation_data_no_donations():
    my_participant = Participant(fake_participant_conf)
    my_participant.output_donation_data()
    assert my_participant._donation_list == []
    fake_participant._format_donation_information_for_output.assert_not_called()
    fake_participant.write_text_files.assert_called()


donation_for_output_json = {"displayName": "Donor 1", "participantID": '4939d', "amount": 34.51,
                            "donorID": "FAKE3C59D235B4DA", "avatarImageURL": "//image.png", "message": "Good job!",
                            "createdDateUTC": "2020-02-11T17:22:23.963+0000", "eventID": 547, "teamID": 50394,
                            "donationID": "861A3C59D235B4DB"}
donation_for_output = eldonationtracker.api.donation.Donation(donation1_json)


@mock.patch.object(eldonationtracker.api.participant.Participant, 'write_text_files', fake_participant.write_text_files)
@mock.patch.object(eldonationtracker.api.participant.Participant, '_format_donation_information_for_output',
                   fake_participant._format_donation_information_for_output)
def test_output_donation_data():
    """Test which functions are called if there is donation data to output."""
    my_participant = Participant(fake_participant_conf)
    my_participant._donation_list = ["a donor", "another_donor"]
    my_participant._top_donation = donation_for_output
    my_participant.output_donation_data()
    fake_participant._format_donation_information_for_output.assert_called()
    fake_participant.write_text_files.assert_called()


@mock.patch.object(eldonationtracker.api.participant.Participant, 'write_text_files', fake_participant.write_text_files)
@mock.patch.object(eldonationtracker.api.participant.Participant, '_format_donation_information_for_output',
                   fake_participant._format_donation_information_for_output)
def test_output_donation_data_no_top_donation():
    """Test which functions are called if there is donation data to ouput."""
    my_participant = Participant(fake_participant_conf)
    my_participant._donation_list = ["a donor", "another_donor"]
    my_participant.output_donation_data()
    fake_participant._format_donation_information_for_output.assert_called()
    fake_participant.write_text_files.assert_called()


@mock.patch.object(eldonationtracker.api.participant.Participant, 'write_text_files', fake_participant.write_text_files)
@mock.patch.object(eldonationtracker.api.participant.Participant, '_format_donor_information_for_output',
                   fake_participant._format_donor_information_for_output)
def test_output_donor_data_no_donations():
    my_participant = Participant(fake_participant_conf)
    my_participant.output_donor_data()
    fake_participant._format_donor_information_for_output.assert_not_called()
    fake_participant.write_text_files.assert_called()


@mock.patch.object(eldonationtracker.api.participant.Participant, 'write_text_files', fake_participant.write_text_files)
@mock.patch.object(eldonationtracker.api.participant.Participant, '_format_donor_information_for_output',
                   fake_participant._format_donor_information_for_output)
def test_output_donor_data():
    my_participant = Participant(fake_participant_conf)
    my_participant._donation_list = ["a donor", "another_donor"]
    my_participant._top_donor = "a top donor"
    my_participant.output_donor_data()
    assert fake_participant._format_donor_information_for_output.called
    assert fake_participant.write_text_files.called


@mock.patch.object(eldonationtracker.api.participant.Participant, 'write_text_files', fake_participant.write_text_files)
@mock.patch.object(eldonationtracker.api.participant.Participant, '_format_donor_information_for_output',
                   fake_participant._format_donor_information_for_output)
def test_output_donor_data_no_top_donor():
    my_participant = Participant(fake_participant_conf)
    my_participant._donation_list = ["a donor", "another_donor"]
    my_participant.output_donor_data()
    assert fake_participant._format_donor_information_for_output.called
    assert fake_participant.write_text_files.called


fake_extralife_io.read_in_total_raised.return_value = ''


@mock.patch.object(eldonationtracker.utils.donor_drive_comms, 'read_in_total_raised', fake_extralife_io.read_in_total_raised)
def test_check_existence_of_text_files_no_value():
    my_participant = Participant(fake_participant_conf)
    is_there_a_text_file = my_participant._check_existence_of_text_files()
    assert is_there_a_text_file is False


fake_extralife_io.read_in_total_raised_low.return_value = '500.00'


@mock.patch.object(eldonationtracker.utils.donor_drive_comms, 'read_in_total_raised',
                   fake_extralife_io.read_in_total_raised_low)
def test_check_existence_of_text_files_low_value():
    my_participant = Participant(fake_participant_conf)
    is_there_a_text_file = my_participant._check_existence_of_text_files()
    assert is_there_a_text_file is True


fake_extralife_io.read_in_total_raised_high.return_value = '1,500.00'


@mock.patch.object(eldonationtracker.utils.donor_drive_comms, 'read_in_total_raised',
                   fake_extralife_io.read_in_total_raised_high)
def test_check_existence_of_text_files_high_value():
    my_participant = Participant(fake_participant_conf)
    is_there_a_text_file = my_participant._check_existence_of_text_files()
    assert is_there_a_text_file is True



fake_output_badge_data = mock.Mock()
fake_output_milestone_data = mock.Mock()


@mock.patch.object(eldonationtracker.api.participant.Participant, 'update_participant_attributes',
                   fake_participant_for_run.update_participant_attributes)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'output_participant_data',
                   fake_participant_for_run.output_participant_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'update_donation_data',
                   fake_participant_for_run.update_donation_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'output_donation_data',
                   fake_participant_for_run.output_donation_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'update_donor_data',
                   fake_participant_for_run.update_donor_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'output_donor_data',
                   fake_participant_for_run.output_donor_data)
@mock.patch.object(eldonationtracker.api.team.Team, 'team_run', fake_participant_for_run.my_team.team_run)
@mock.patch.object(eldonationtracker.api.team.Team, 'participant_run',
                   fake_participant_for_run.my_team.participant_run)
@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "get_badges", magic_fake_badges.get_badges)
@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "output_badge_data", fake_output_badge_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, '_update_milestones',
                   fake_participant_for_run._update_milestones)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'output_milestone_data',
                   fake_participant_for_run.output_milestone_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, '_update_incentives',
                   fake_participant_for_run.fake_participant_for_run._update_incentives)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'output_incentive_data',
                   fake_participant_for_run.output_incentive_data)
def test_run():
    my_participant = Participant(fake_participant_conf)
    assert my_participant.number_of_donations == 0
    assert my_participant._first_run
    my_participant._goal = 500  # needed to make below run - will probably be fixed with better tests in future
    my_participant.run()
    fake_participant_for_run.update_participant_attributes.assert_called_once()
    fake_participant_for_run.output_participant_data.assert_called_once()
    fake_participant_for_run.update_donation_data.assert_called_once()
    fake_participant_for_run.output_donation_data.assert_called_once()
    fake_participant_for_run.update_donor_data.assert_called_once()
    fake_participant_for_run.my_team.team_run.assert_called_once()
    assert my_participant._first_run is False
    my_participant.run()
    assert fake_participant_for_run.update_participant_attributes.call_count == 2
    assert fake_participant_for_run.output_participant_data.call_count == 2
    fake_participant_for_run.update_donation_data.assert_called_once()
    fake_participant_for_run.output_donation_data.assert_called_once()
    fake_participant_for_run.update_donor_data.assert_called_once()
    assert fake_participant_for_run.my_team.team_run.call_count == 2


@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "get_json", fake_extralife_io.get_json)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'output_participant_data',
                   fake_participant_for_run.output_participant_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'update_donation_data',
                   fake_participant_for_run.update_donation_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'output_donation_data',
                   fake_participant_for_run.output_donation_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'update_donor_data',
                   fake_participant_for_run.update_donor_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'output_donor_data',
                   fake_participant_for_run.output_donor_data)
@mock.patch.object(eldonationtracker.api.team.Team, 'team_run', fake_participant_for_run.my_team.team_run)
@mock.patch.object(eldonationtracker.api.team.Team, 'participant_run',
                   fake_participant_for_run.my_team.participant_run)
@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "get_badges", magic_fake_badges.get_badges)
@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "output_badge_data", fake_output_badge_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, '_update_milestones',
                   fake_participant_for_run._update_milestones)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'output_milestone_data',
                   fake_participant_for_run.output_milestone_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, '_update_incentives',
                   fake_participant_for_run.fake_participant_for_run._update_incentives)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'output_incentive_data',
                   fake_participant_for_run.output_incentive_data)
def test_run_get_a_donation():
    fake_participant_for_run.output_participant_data.reset_mock()
    fake_participant_for_run.update_donation_data.reset_mock()
    fake_participant_for_run.output_donation_data.reset_mock()
    fake_participant_for_run.update_donor_data.reset_mock()
    fake_participant_for_run.output_donor_data.reset_mock()
    fake_participant_for_run.my_team.team_run.reset_mock()
    fake_participant_for_run.my_team.participant_run.reset_mock()
    my_participant = Participant(fake_participant_conf)
    assert my_participant.number_of_donations == 0
    my_participant._first_run = False  # to simulate that this is after one run already
    my_participant._donor_list = ['this is fake mocked and does not matter']
    my_participant.run()
    fake_participant_for_run.output_participant_data.assert_called_once()
    fake_participant_for_run.update_donation_data.assert_called_once()
    fake_participant_for_run.output_donation_data.assert_called_once()
    fake_participant_for_run.update_donor_data.assert_called_once()
    fake_participant_for_run.output_donor_data.assert_called_once()
    fake_participant_for_run.my_team.team_run.assert_called_once()


@mock.patch.object(eldonationtracker.api.participant.Participant, 'write_text_files', fake_participant.write_text_files)
@mock.patch.object(eldonationtracker.api.team.Team, 'team_run', fake_participant_for_run.my_team.team_run)
@mock.patch.object(eldonationtracker.api.team.Team, 'participant_run',
                   fake_participant_for_run.my_team.participant_run)
@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "get_badges", magic_fake_badges.get_badges)
@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "output_badge_data", fake_output_badge_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, '_update_milestones',
                   fake_participant_for_run._update_milestones)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'output_milestone_data',
                   fake_participant_for_run.output_milestone_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, '_update_incentives',
                   fake_participant_for_run.fake_participant_for_run._update_incentives)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'output_incentive_data',
                   fake_participant_for_run.output_incentive_data)
def test_run_no_api_hit():
    """Making sure that output data is not hit if api wasn't hit."""
    fake_participant_for_run.output_participant_data.reset_mock()
    my_participant = Participant(fake_participant_conf)
    my_participant.run()
    fake_participant_for_run.output_participant_data.assert_not_called()


@mock.patch.object(eldonationtracker.api.participant.Participant, 'write_text_files', fake_participant.write_text_files)
@mock.patch.object(eldonationtracker.api.team.Team, 'team_run', fake_participant_for_run.my_team.team_run)
@mock.patch.object(eldonationtracker.api.team.Team, 'participant_run',
                   fake_participant_for_run.my_team.participant_run)
@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "get_badges", magic_fake_badges.get_badges)
@mock.patch.object(eldonationtracker.utils.donor_drive_comms, "output_badge_data", fake_output_badge_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, '_update_milestones',
                   fake_participant_for_run._update_milestones)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'output_milestone_data',
                   fake_participant_for_run.output_milestone_data)
@mock.patch.object(eldonationtracker.api.participant.Participant, '_update_incentives',
                   fake_participant_for_run.fake_participant_for_run._update_incentives)
@mock.patch.object(eldonationtracker.api.participant.Participant, 'output_incentive_data',
                   fake_participant_for_run.output_incentive_data)
def test_run_no_api_hit_no_team():
    """Making sure that output data is not hit if api wasn't hit."""
    fake_participant_for_run.output_participant_data.reset_mock()
    fake_participant_for_run.my_team.team_run.reset_mock()
    fake_participant_for_run.my_team.participant_run.reset_mock()
    my_participant = Participant(fake_participant_conf_no_team)
    my_participant.run()
    fake_participant_for_run.output_participant_data.assert_not_called()
    fake_participant_for_run.my_team.team_run.assert_not_called()



"""Unit tests for team.py."""

from unittest import mock

from eldonationtracker.api import team as team


def test_team_urls():
    """Test that URL construction is correct."""
    my_team = team.Team("12345", "folder", "$", "5")
    assert my_team.team_url == "https://www.extra-life.org/api/teams/12345"
    assert my_team.team_participant_url == "https://www.extra-life.org/api/teams/12345/participants"
    assert my_team.team_donation_url == "https://www.extra-life.org/api/teams/12345/donations"


def test_output_folder():
    """Test that the program correctly parses the output folder."""
    my_team = team.Team("12345", "folder", "$", "5")
    assert my_team.output_folder == "folder"


def test_donors_to_display():
    """Test that the program correct parses the number of donors to display."""
    my_team = team.Team("12345", "folder", "$", "5")
    assert my_team.donors_to_display == "5"


def test_get_team_json():
    """Test that the JSON we get back is correctly interpreted."""
    with mock.patch("eldonationtracker.api.team.extralife_io.get_json",
                    return_value={"fundraisingGoal": 500, "captainDisplayName": "Captain Awesome",
                                  "sumDonations": 400, "numDonations": 300,
                                  "avatarImageURL": "//assets.donordrive.com/…avatar-team-default.gif"}):
        my_team = team.Team("12345", "folder", "$", "5")
        team_json = my_team._get_team_json()
        assert team_json == (500, 'Captain Awesome', 400, 300, "//assets.donordrive.com/…avatar-team-default.gif")


def test_get_team_json_no_json():
    """Test to make sure if no JSON is returned from the endpoint, the program responds correcty."""
    with mock.patch("eldonationtracker.api.team.extralife_io.get_json", return_value={}):
        my_team = team.Team("12345", "folder", "$", "5")
        team_json = my_team._get_team_json()
        assert team_json == (0, '', 0, 0, '')
        # let's pretend that at some point values were added
        # but now the api can't be reached. Let's make sure it doesn't over-write the good data.
        my_team._team_goal = 500
        my_team._team_captain = 'Captain Awesome'
        my_team._total_raised = 400
        my_team._num_donations = 300
        my_team._team_avatar_image = "//assets.donordrive.com/…avatar-team-default.gif"
        team_json = my_team._get_team_json()
        assert team_json == (500, 'Captain Awesome', 400, 300, "//assets.donordrive.com/…avatar-team-default.gif")


def test_update_team_dictionary():
    my_team = team.Team("12345", "folder", "$", "5")
    my_team._team_goal = 500
    my_team._team_captain = 'Captain Awesome'
    my_team._total_raised = 400
    my_team._num_donations = 300
    my_team._update_team_dictionary()
    assert list(my_team._team_info.values()) == ['$500.00', 'Captain Awesome', '$400.00', '300']


def test_get_participants_no_participants():
    my_team = team.Team("12345", "folder", "$", "5")
    with mock.patch("eldonationtracker.api.team.extralife_io.get_json", return_value={}):
        participants = my_team._get_participants(False)
        assert participants == []
        assert my_team._participant_list == []


def test_get_participants():
    my_team = team.Team("12345", "folder", "$", "5")
    team_participants = [{"displayName":"Karl Abraham",
                          "fundraisingGoal":500.00,
                          "eventName":"Extra Life 2020",
                          "links":{"donate":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411130#donate",
                                   "stream":"https://player.twitch.tv/?channel=ToxicAntidote",
                                   "page":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411130"},
                          "createdDateUTC":"2020-05-31T23:08:31.103+0000","eventID":547,"sumDonations":0.00,
                          "participantID":411130,"teamName":"Giant Bomb",
                          "avatarImageURL":"//assets.donordrive.com/extralife/images/$avatars$/constituent_18EE04BD-BBCC-439D-B1DC6C5B9FCD9130.jpg",
                          "streamIsLive":False,"teamID":50394,"isTeamCaptain":False,"sumPledges":0.00,"numDonations":0},
                         {"displayName":"Ben Tolmachoff","fundraisingGoal":500.00,"eventName":"Extra Life 2020",
                          "links":{"donate":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411107#donate",
                                   "stream":"https://player.twitch.tv/?channel=chof",
                                   "page":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411107"},
                          "createdDateUTC":"2020-05-30T21:34:38.133+0000","eventID":547,"sumDonations":0.00,
                          "participantID":411107,"teamName":"Giant Bomb",
                          "avatarImageURL":"//assets.donordrive.com/extralife/images/$avatars$/constituent_FACE6F26-A2A6-80BB-F35771F8733FBEBE.jpg",
                          "streamIsLive":False,"teamID":50394,"isTeamCaptain":False,"sumPledges":0.00,"numDonations":0},
                         {"displayName":"Michael Bataligin","fundraisingGoal":100.00,"eventName":"Extra Life 2020",
                          "links":{"donate":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=410654#donate",
                                   "page":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=410654"},
                          "createdDateUTC":"2020-05-23T00:19:14.707+0000","eventID":547,"sumDonations":5.00,
                          "participantID":410654,"teamName":"Giant Bomb",
                          "avatarImageURL":"//assets.donordrive.com/extralife/images/$avatars$/constituent_0AFEA929-C29F-F29A-6B659B3718802B75.jpg",
                          "teamID":50394,"isTeamCaptain":False,"sumPledges":0.00,"numDonations":1}]
    with mock.patch("eldonationtracker.api.team.extralife_io.get_json", return_value=team_participants):
        participants = my_team._get_participants(False)
        assert participants[0].name == "Karl Abraham"
        assert participants[1].name == "Ben Tolmachoff"


def test_get_participants_no_participants_top_5():
    my_team = team.Team("12345", "folder", "$", "5")
    with mock.patch("eldonationtracker.api.team.extralife_io.get_json", return_value={}):
        participants = my_team._get_participants(True)
        assert participants == []
        assert my_team._top_5_participant_list == []


def test_get_participants_top_5():
    """Test getting top 5 participants.

    While, of course, in reality the JSON would be different - sorted by donations, in terms
    of what the code has to do with what it gets back, there is no difference. So I have just copied things
    to ensure I'm considering both parts of the if statement in this function.
    """
    my_team = team.Team("12345", "folder", "$", "5")
    team_participants = [{"displayName":"Karl Abraham",
                          "fundraisingGoal":500.00,
                          "eventName":"Extra Life 2020",
                          "links":{"donate":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411130#donate",
                                   "stream":"https://player.twitch.tv/?channel=ToxicAntidote",
                                   "page":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411130"},
                          "createdDateUTC":"2020-05-31T23:08:31.103+0000","eventID":547,"sumDonations":0.00,
                          "participantID":411130,"teamName":"Giant Bomb",
                          "avatarImageURL":"//assets.donordrive.com/extralife/images/$avatars$/constituent_18EE04BD-BBCC-439D-B1DC6C5B9FCD9130.jpg",
                          "streamIsLive":False,"teamID":50394,"isTeamCaptain":False,"sumPledges":0.00,"numDonations":0},
                         {"displayName":"Ben Tolmachoff","fundraisingGoal":500.00,"eventName":"Extra Life 2020",
                          "links":{"donate":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411107#donate",
                                   "stream":"https://player.twitch.tv/?channel=chof",
                                   "page":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411107"},
                          "createdDateUTC":"2020-05-30T21:34:38.133+0000","eventID":547,"sumDonations":0.00,
                          "participantID":411107,"teamName":"Giant Bomb",
                          "avatarImageURL":"//assets.donordrive.com/extralife/images/$avatars$/constituent_FACE6F26-A2A6-80BB-F35771F8733FBEBE.jpg",
                          "streamIsLive":False,"teamID":50394,"isTeamCaptain":False,"sumPledges":0.00,"numDonations":0},
                         {"displayName":"Michael Bataligin","fundraisingGoal":100.00,"eventName":"Extra Life 2020",
                          "links":{"donate":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=410654#donate",
                                   "page":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=410654"},
                          "createdDateUTC":"2020-05-23T00:19:14.707+0000","eventID":547,"sumDonations":5.00,
                          "participantID":410654,"teamName":"Giant Bomb",
                          "avatarImageURL":"//assets.donordrive.com/extralife/images/$avatars$/constituent_0AFEA929-C29F-F29A-6B659B3718802B75.jpg",
                          "teamID":50394,"isTeamCaptain":False,"sumPledges":0.00,"numDonations":1}]
    with mock.patch("eldonationtracker.api.team.extralife_io.get_json", return_value=team_participants):
        participants = my_team._get_participants(True)
        assert participants[0].name == "Karl Abraham"
        assert participants[1].name == "Ben Tolmachoff"


def test_top_participant_no_participants():
    my_team = team.Team("12345", "folder", "$", "5")
    top_participant = my_team._top_participant()
    assert top_participant == "No participants."


def test_top_participant():
    my_team = team.Team("12345", "folder", "$", "5")
    team_participants = [{"displayName":"Karl Abraham",
                          "fundraisingGoal":500.00,
                          "eventName":"Extra Life 2020",
                          "links":{"donate":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411130#donate",
                                   "stream":"https://player.twitch.tv/?channel=ToxicAntidote",
                                   "page":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411130"},
                          "createdDateUTC":"2020-05-31T23:08:31.103+0000","eventID":547,"sumDonations":0.00,
                          "participantID":411130,"teamName":"Giant Bomb",
                          "avatarImageURL":"//assets.donordrive.com/extralife/images/$avatars$/constituent_18EE04BD-BBCC-439D-B1DC6C5B9FCD9130.jpg",
                          "streamIsLive":False,"teamID":50394,"isTeamCaptain":False,"sumPledges":0.00,"numDonations":0},
                         {"displayName":"Ben Tolmachoff","fundraisingGoal":500.00,"eventName":"Extra Life 2020",
                          "links":{"donate":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411107#donate",
                                   "stream":"https://player.twitch.tv/?channel=chof",
                                   "page":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411107"},
                          "createdDateUTC":"2020-05-30T21:34:38.133+0000","eventID":547,"sumDonations":0.00,
                          "participantID":411107,"teamName":"Giant Bomb",
                          "avatarImageURL":"//assets.donordrive.com/extralife/images/$avatars$/constituent_FACE6F26-A2A6-80BB-F35771F8733FBEBE.jpg",
                          "streamIsLive":False,"teamID":50394,"isTeamCaptain":False,"sumPledges":0.00,"numDonations":0},
                         {"displayName":"Michael Bataligin","fundraisingGoal":100.00,"eventName":"Extra Life 2020",
                          "links":{"donate":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=410654#donate",
                                   "page":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=410654"},
                          "createdDateUTC":"2020-05-23T00:19:14.707+0000","eventID":547,"sumDonations":5.00,
                          "participantID":410654,"teamName":"Giant Bomb",
                          "avatarImageURL":"//assets.donordrive.com/extralife/images/$avatars$/constituent_0AFEA929-C29F-F29A-6B659B3718802B75.jpg",
                          "teamID":50394,"isTeamCaptain":False,"sumPledges":0.00,"numDonations":1}]
    with mock.patch("eldonationtracker.api.team.extralife_io.get_json", return_value=team_participants):
        my_team._top_5_participant_list = my_team._get_participants(True)
        top_participant = my_team._top_participant()
        assert top_participant == "Karl Abraham - $0.00"


def test_participant_calculations_no_data():
    """What if the api comes back empty? Maybe a team just formed without any members?"""
    my_team = team.Team("12345", "folder", "$", "5")
    my_team._participant_calculations()
    assert my_team._participant_calculation_dict['Team_TopParticipantNameAmnt'] == "No participants."
    assert my_team._participant_calculation_dict['Team_Top5ParticipantsHorizontal'] == ""
    assert my_team._participant_calculation_dict['Team_Top5Participants'] == ""


def test_participant_calculations():
    my_team = team.Team("12345", "folder", "$", "5")
    team_participants = [{"displayName":"Karl Abraham",
                          "fundraisingGoal":500.00,
                          "eventName":"Extra Life 2020",
                          "links":{"donate":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411130#donate",
                                   "stream":"https://player.twitch.tv/?channel=ToxicAntidote",
                                   "page":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411130"},
                          "createdDateUTC":"2020-05-31T23:08:31.103+0000","eventID":547,"sumDonations":0.00,
                          "participantID":411130,"teamName":"Giant Bomb",
                          "avatarImageURL":"//assets.donordrive.com/extralife/images/$avatars$/constituent_18EE04BD-BBCC-439D-B1DC6C5B9FCD9130.jpg",
                          "streamIsLive":False,"teamID":50394,"isTeamCaptain":False,"sumPledges":0.00,"numDonations":0},
                         {"displayName":"Ben Tolmachoff","fundraisingGoal":500.00,"eventName":"Extra Life 2020",
                          "links":{"donate":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411107#donate",
                                   "stream":"https://player.twitch.tv/?channel=chof",
                                   "page":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=411107"},
                          "createdDateUTC":"2020-05-30T21:34:38.133+0000","eventID":547,"sumDonations":0.00,
                          "participantID":411107,"teamName":"Giant Bomb",
                          "avatarImageURL":"//assets.donordrive.com/extralife/images/$avatars$/constituent_FACE6F26-A2A6-80BB-F35771F8733FBEBE.jpg",
                          "streamIsLive":False,"teamID":50394,"isTeamCaptain":False,"sumPledges":0.00,"numDonations":0},
                         {"displayName":"Michael Bataligin","fundraisingGoal":100.00,"eventName":"Extra Life 2020",
                          "links":{"donate":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=410654#donate",
                                   "page":"https://www.extra-life.org/index.cfm?fuseaction=donorDrive.participant&participantID=410654"},
                          "createdDateUTC":"2020-05-23T00:19:14.707+0000","eventID":547,"sumDonations":5.00,
                          "participantID":410654,"teamName":"Giant Bomb",
                          "avatarImageURL":"//assets.donordrive.com/extralife/images/$avatars$/constituent_0AFEA929-C29F-F29A-6B659B3718802B75.jpg",
                          "teamID":50394,"isTeamCaptain":False,"sumPledges":0.00,"numDonations":1}]
    with mock.patch("eldonationtracker.api.team.extralife_io.get_json", return_value=team_participants):
        my_team._top_5_participant_list = my_team._get_participants(True)
        my_team._participant_calculations()
        assert my_team._participant_calculation_dict['Team_TopParticipantNameAmnt'] == "Karl Abraham - $0.00"
        assert my_team._participant_calculation_dict['Team_Top5ParticipantsHorizontal'] == "Karl Abraham - $0.00 | Ben Tolmachoff - $0.00 | Michael Bataligin - $5.00 | "
        assert my_team._participant_calculation_dict['Team_Top5Participants'] == "Karl Abraham - $0.00\nBen Tolmachoff - $0.00\nMichael Bataligin - $5.00\n"


fake_get_team_json = mock.Mock()
fake_get_team_json.return_value = 400, "Captain", 401, 3, "//assets.donordrive.com/…avatar-team-default.gif"
fake_participant_run = mock.Mock()
fake_write_text_files = mock.Mock()
fake_donation_run = mock.Mock()
fake_badge_run = mock.Mock()


@mock.patch.object(team.Team, "_get_team_json", fake_get_team_json)
@mock.patch.object(team.Team, "participant_run", fake_participant_run)
@mock.patch.object(team.Team, "write_text_files", fake_write_text_files)
@mock.patch.object(team.Team, "donation_run", fake_donation_run)
def test_team_run():
    my_team = team.Team("12345", "folder", "$", "5")
    my_team.team_run()
    assert fake_participant_run.call_count == 1
    my_team.team_run()
    assert fake_participant_run.call_count == 1
    fake_get_team_json.return_value = 400, "Captain", 402, 4, "//assets.donordrive.com/…avatar-team-default.gif"
    my_team.team_run()
    assert fake_participant_run.call_count == 2


fake_get_team_json2 = mock.Mock()
fake_get_team_json2.return_value = 400, "Captain", 401, 3, "//assets.donordrive.com/…avatar-team-default.gif"
fake_output_badge_data = mock.Mock()


@mock.patch.object(team.Team, "_get_team_json", fake_get_team_json2)
@mock.patch.object(team.Team, "participant_run", fake_participant_run)
@mock.patch.object(team.Team, "write_text_files", fake_write_text_files)
@mock.patch.object(team.Team, "donation_run", fake_donation_run)
@mock.patch.object(team.Team, "_update_badges", fake_badge_run)
@mock.patch.object(team.donor_drive_comms, "output_badge_data", fake_output_badge_data)
def test_team_api_info():
    my_team = team.Team("12345", "folder", "$", "5")
    my_team.team_api_info()
    assert my_team.team_goal == 400
    assert my_team.team_captain == "Captain"
    assert my_team.total_raised == 401
    assert my_team.num_donations == 3
    assert my_team._team_info["Team_goal"] == '$400.00'
    assert my_team._team_info["Team_captain"] == "Captain"
    assert my_team._team_info["Team_totalRaised"] == "$401.00"
    assert my_team._team_info["Team_numDonations"] == '3'


def test_str_no_json_data():
    """Test what str will produce if the JSON retrieval hasn't yet run."""
    my_team = team.Team("12345", "folder", "$", "5")
    assert str(my_team) == "A team found at https://www.extra-life.org/api/teams/12345. "


def test_str():
    """Test what str will produce after team_info has been filled out."""
    my_team = team.Team("12345", "folder", "$", "5")
    my_team._team_info["Team_goal"] = '$400.00'
    assert str(my_team) == "A team found at https://www.extra-life.org/api/teams/12345. Team goal is $400.00."


def test_str_no_json_data_no_team_id():
    """Test what str will produce if the JSON retrieval hasn't yet run AND no team ID.

    This would happen if the user is not part of a team.
    """
    my_team = team.Team(None, "folder", "$", "5")
    assert str(my_team) == "Not a valid team - no team_id."



from donordrivepython.api import donor as donor


class TeamParticipant(donor.Donor):
    """Participant Attributes.

    Inherits from the donor class, but
    over-rides the json_to_attributes function.

    api variables:

    :param self.name: participant's name
    :param self.amount: the sum of all donations by this participant
    :param self.number_of_donations: number of all donations by this participant
    :param self.image_url: the url of the participant's avatar image (not used)
    """

    def __str__(self):
        return f"A Team Participant named {self.name} who has donated ${self.amount:.2f} to the team over" \
               f" {self.number_of_donations} donations."



"""Test methods in donor class."""

from eldonationtracker.api import donation

donation1_json = {"displayName": "Donor 1", "participantID": '4939d', "amount": 34.51, "donorID": "FAKE3C59D235B4DA",
                  "avatarImageURL": "//image.png", "message": "Good job!",
                  "createdDateUTC": "2020-02-11T17:22:23.963+0000", "eventID": 547, "teamID": 50394,
                  "donationID": "861A3C59D235B4DB"}
donation1 = donation.Donation(donation1_json)

donation2_json = {"displayName": "Donor 1", "participantID": '4939d', "amount": 34.50, "donorID": "FAKE3C59D235B4DA",
                  "avatarImageURL": "//image.png",
                  "createdDateUTC": "2020-02-11T17:22:23.963+0000", "eventID": 547, "teamID": 50394,
                  "donationID": "861A3C59D235B4DC"}
donation2 = donation.Donation(donation2_json)
donation2_equal = donation.Donation(donation1_json)

donation_anonymous_json = {"displayName": None, "participantID": '4939d', "amount": 34.51, "message": "Good job!",
                           "donorID": "FAKE3C59D235B4DA", "avatarImageURL": "//image.png",
                           "createdDateUTC": "2020-02-11T17:22:23.963+0000", "eventID": 547, "teamID": 50394,
                           "donationID": "861A3C59D235B4DD"}
donation_anonymous = donation.Donation(donation_anonymous_json)

donation_no_money_json = {"displayName": "Donor 1", "participantID": '4939d', "amount": None, "message": "Good job!",
                          "donorID": "FAKE3C59D235B4DA", "avatarImageURL": "//image.png",
                          "createdDateUTC": "2020-02-11T17:22:23.963+0000", "eventID": 547, "teamID": 50394,
                          "donationID": "861A3C59D235B4DE"}
donation_no_money = donation.Donation(donation_no_money_json)

donation1_whole_dollar_json = {"displayName": "Donor 1", "participantID": '4939d', "amount": 34,
                               "donorID": "FAKE3C59D235B4DA", "avatarImageURL": "//image.png",
                               "createdDateUTC": "2020-02-11T17:22:23.963+0000", "eventID": 547, "teamID": 50394,
                               "donationID": "861A3C59D235B4DF"}
donation1_whole_dollar = donation.Donation(donation1_whole_dollar_json)

donation2_whole_dollar_json = {"displayName": "Donor 1", "participantID": '4939d', "amount": 32,
                               "donorID": "FAKE3C59D235B4DA", "avatarImageURL": "//image.png",
                               "createdDateUTC": "2020-02-11T17:22:23.963+0000", "eventID": 547, "teamID": 50394,
                               "donationID": "861A3C59D235B4DG"}
donation2_whole_dollar = donation.Donation(donation2_whole_dollar_json)

fake_participant_info = {'sumDonations': 500, 'numDonations': 5, 'fundraisingGoal': 1000}
fake_donations = {"displayName": "Sean Gibson", "participantID": 401280, "amount": 25.00, "donorID": "54483486D840B7EA",
                  "avatarImageURL": "//assets.donordrive.com/clients/extralife/img/avatar-constituent-default.gif",
                  "createdDateUTC": "2020-02-11T17:22:23.963+0000", "eventID": 547, "teamID": 50394,
                  "donationID": "861A3C59D235B4DA"}, {"displayName": "Eric Mesa", "participantID": 401280,
                                                      "amount": 25.00, "donorID": "4162ECD2B8BF4C17",
                                                      "avatarImageURL": "//assets.donordrive.com/extralife/images/$avat"
                                                                        "ars$/constituent_D4DC394A-C293-34EB-4162ECD2B"
                                                                        "8BF4C17.jpg",
                                                      "createdDateUTC": "2020-01-05T20:35:28.897+0000",
                                                      "eventID": 547, "teamID": 50394, "donationID": "7D430E9E9AF79686"}
donation_emoji_json = {"displayName": "🍀🍀🍀🍀", "participantID": '4939d', "amount": 34.51,
                       "donorID": "FAKE3C59D235B4DA", "avatarImageURL": "//image.png", "message": "Good job!",
                       "createdDateUTC": "2020-02-11T17:22:23.963+0000", "eventID": 547, "teamID": 50394,
                       "donationID": "861A3C59D235B4DA"}


def test_donation_attributes_no_blanks():
    """Test to make sure attributes are properly assigned."""
    assert donation1.name == "Donor 1"
    assert donation1.message == "Good job!"
    assert donation1.amount == 34.51


def test_donation_attributes_no_name():
    """Test to make sure attributes are properly assigned."""
    assert donation_anonymous.name == "Anonymous"
    assert donation_anonymous.message == "Good job!"
    assert donation_anonymous.amount == 34.51


def test_donation_attributes_no_amount():
    """Test to make sure attributes are properly assigned."""
    assert donation_no_money.name == "Donor 1"
    assert donation_no_money.message == "Good job!"
    assert donation_no_money.amount == 0


def test_donation_lt_whole_numbers():
    """Test to make sure comparison works.

    Uses whole dollar amounts.
    """
    assert donation2_whole_dollar.amount < donation1_whole_dollar.amount


def test_donation_lt_change():
    """Test to make sure comparison works.

    Uses dollars and cents.
    """
    assert donation2.amount < donation1.amount


def test_donation_equal():
    """Test to make sure comparison works.

    This time they're equal.
    """
    assert donation2_equal.amount == donation1.amount


def test_donation_str():
    """Test that the class string is properly created."""
    assert str(donation1) == "A donation by Donor 1 in the amount of $34.51 with the message 'Good job!'"


def test_emoji_in_donor_name():
    """Test that an emoji in the donor name will not crash things."""
    emoji_donation = donation.Donation(donation_emoji_json)
    assert emoji_donation.name == "🍀🍀🍀🍀"
    assert str(emoji_donation) == "A donation by 🍀🍀🍀🍀 in the amount of $34.51 with the message 'Good job!'"


"""A donor."""


class Donor:
    """Donor Attributes.

    Class exists to provide attributes for a donor based on what comes in from
    the JSON so that it doesn't have to be traversed each time a donor action
    needs to be taken.
    """

    def __init__(self, json):
        """Load in values from class initialization.


        :param json: JSON attributes from the api
        :type json: json
        """
        self._name, self._donor_id, self._image_url, self._amount, self._number_of_donations =\
            self.json_to_attributes(json)

    @property
    def name(self) -> str:
        """Donor's name if provided, else Anonymous."""
        return self._name

    @property
    def donor_id(self) -> str:
        """The ID assigned by the api (currently not used)."""
        return self._donor_id

    @property
    def image_url(self) -> str:
        """The URL for the donor's avatar (currently not used)"""
        return self._image_url

    @property
    def amount(self) -> float:
        """The sum of all donations the donor has made this campaign"""
        return self._amount

    @property
    def number_of_donations(self) -> str:
        """The number of donations the donor has made this campaign"""
        return self._number_of_donations

    @staticmethod
    def json_to_attributes(json_values):
        """Convert api JSON values to Donor attributes.

        :param json_values: JSON attributes from the api
        :type json_values: json
        """
        if json_values.get('displayName') is not None:
            name = json_values.get('displayName')
        else:
            name = "Anonymous"
        donor_id = json_values.get('donorID')
        image_url = json_values.get('avatarImageURL')
        amount = float(json_values.get('sumDonations'))
        number_of_donations = json_values.get('numDonations')
        return name, donor_id, image_url, amount, number_of_donations

    def __lt__(self, other):
        """Donor less than comparison.

        Returns True if this Donor has a donation amount less than comparison.
        """
        return self.amount < other.amount

    def __eq__(self, other):
        """Donor equal comparison.

        Returns True if this Donor and the other have the same amount.
        """
        return self.amount == other.amount

    def __str__(self):
        """Returns a string representation of this donor."""
        return f"A donor named {self.name} with donor ID {self.donor_id} who has donated ${self.amount:.2f}"\
               f" over {self.number_of_donations} donations."



"""A Teamgorup is a group of teams, naturally."""
from dataclasses import dataclass

@dataclass
class TeamGroup:
    fundraising_goal: float
    group_code: str
    name: str
    number_of_donations: int
    number_of_participants: int
    number_of_teams: int
    sum_of_donations: float


def create_team_group(json_data) -> TeamGroup:
    fundraising_goal: float = json_data.get('fundraisingGoal')
    group_code: str = json_data.get('groupCode')
    name: str = json_data.get('name')
    number_of_donations: int = json_data.get('numDonations')
    number_of_participants: int = json_data.get('numParticipants')
    number_of_teams: int = json_data.get('numTeams')
    sum_of_donations: float = json_data.get('sumDonations')
    return TeamGroup(fundraising_goal, group_code, name, number_of_donations, number_of_participants, number_of_teams,
                     sum_of_donations)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['donordrivepython', 'donordrivepython.api', 'donordrivepython.api.tests']

package_data = \
{'': ['*']}

install_requires = \
['requests==2.27.1',
 'rich>=12.4.0,<13.0.0',
 'tzlocal>=4.2,<5.0',
 'xdgenvpy>=2.3.5,<3.0.0']

setup_kwargs = {
    'name': 'donordrivepython',
    'version': '1.4.1',
    'description': 'A utility to access the DonorDrive API',
    'long_description': 'DonorDrivePython\n----------------\n\nA Python reference implementation of the [Donor Drive API](https://github.com/DonorDrive/PublicAPI)\n\nThis project aims to provide a Python package the user could import to create a project to access the Donor Drive API.\n\nFor an example of what you can build to provide extra functionality around the Donor Drive API, see my project, [ElDonationTracker](http://djotaku.github.io/ELDonationTracker/) for the Extra Life charity event. It takes the Donor Drive API information and converts it to text files and HTML files that the gamers can use for live streaming during the Extra Life Event. Currently, it has the API integrated into it. The API implementation there will become the initial release in this repo.\n',
    'author': 'Eric Mesa',
    'author_email': 'ericsbinaryworld@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/djotaku/DonorDrivePython',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



"""Unit tests for team_participant.py"""

import eldonationtracker.api.team_participant


def test_team_participant_str():
    team_donor1_json = {"displayName": "donor1", "sumDonations": "45", "donorID": 1000111,
                        'avatarImageURL': "http://someplace.com/image.jpg", "numDonations": 2}
    team_donor1 = eldonationtracker.api.team_participant.TeamParticipant(team_donor1_json)
    assert str(team_donor1) == "A Team Participant named donor1 who has donated $45.00 to the team over 2 donations."


def test_team_participant_str_no_donor_name():
    team_donor1_json = {"sumDonations": "45", "donorID": 1000111,
                        'avatarImageURL': "http://someplace.com/image.jpg", "numDonations": 2}
    team_donor1 = eldonationtracker.api.team_participant.TeamParticipant(team_donor1_json)
    assert str(team_donor1) == "A Team Participant named Anonymous who has donated $45.00 to the team over 2 donations."
